@@ -25,13 +25,6 @@
 /* moved common code to library file */
 #include "openswan/passert.h"
 
-#define loglog  openswan_loglog
-#define plog    openswan_log
-extern int openswan_log(const char *message, ...) PRINTF_LIKE(1);
-extern void openswan_loglog(int mess_no, const char *message, ...) PRINTF_LIKE(2);
-extern void openswan_exit_log(const char *message, ...) PRINTF_LIKE(1);
-
-
 #if !defined(NO_DEBUG)
 
 #include "constants.h"
@@ -43,9 +36,14 @@ extern lset_t cur_debugging;	/* current debugging level */
 #define DBG(cond, action)   { if (DBGP(cond)) { action ; } }
 
 #define DBG_log openswan_DBG_log
+#define loglog  openswan_loglog
+#define plog    openswan_log
 #define DBG_dump openswan_DBG_dump
 extern int openswan_DBG_log(const char *message, ...) PRINTF_LIKE(1);
 extern void openswan_DBG_dump(const char *label, const void *p, size_t len);
+extern int openswan_log(const char *message, ...) PRINTF_LIKE(1);
+extern void openswan_loglog(int mess_no, const char *message, ...) PRINTF_LIKE(2);
+extern void openswan_exit_log(const char *message, ...) PRINTF_LIKE(1);
 
 #define DBG_dump_chunk(label, ch) DBG_dump(label, (ch).ptr, (ch).len)
 
@@ -58,18 +56,21 @@ extern void set_debugging(lset_t deb);
 #else /*!DEBUG*/
 
 #define DBG(cond, action)	{ }	/* do nothing */
-#define DBGP(...) (0)
-#define exit_tool exit
-#define openswan_DBG_dump(...) do { } while(0)
-#define DBG_log(...) do { } while(0)
-extern void tool_init_log(void);
-extern void tool_close_log(void);
 
 #endif /*!DEBUG*/
 
 #define DBG_cond_dump(cond, label, p, len) DBG(cond, DBG_dump(label, p, len))
 #define DBG_cond_dump_chunk(cond, label, ch) DBG(cond, DBG_dump_chunk(label, ch))
 
+/*
+ * function to log stuff from libraries that may be used in multiple
+ * places.
+ */
+#ifndef OPENSWAN_KEYING_FUNCTION 
+#define OPENSWAN_KEYING_FUNCTION 
+typedef int (*openswan_keying_debug_func_t)(const char *message, ...);
+#endif
+
 /* Build up a diagnostic in a static buffer.
  * Although this would be a generally useful function, it is very
  * hard to come up with a discipline that prevents different uses
