@@ -61,7 +61,7 @@
 
 #endif /* __KERNEL__ */
 
-#define DEBUG_NO_STATIC static
+#  define DEBUG_NO_STATIC static
 #include <openswan/ipsec_kversion.h>
 #include <openswan/ipsec_param.h>
 #endif /* linux */
@@ -247,9 +247,10 @@ typedef uint32_t IPsecSAref_t;
  * function to log stuff from libraries that may be used in multiple
  * places.
  */
+#ifndef OPENSWAN_KEYING_FUNCTION 
+#define OPENSWAN_KEYING_FUNCTION 
 typedef int (*openswan_keying_debug_func_t)(const char *message, ...);
-
-
+#endif
 
 /*
  * new IPv6-compatible functions
@@ -259,13 +260,7 @@ typedef int (*openswan_keying_debug_func_t)(const char *message, ...);
 err_t ttoul(const char *src, size_t srclen, int format, unsigned long *dst);
 size_t ultot(unsigned long src, int format, char *buf, size_t buflen);
 #define	ULTOT_BUF	(22+1)	/* holds 64 bits in octal */
-
-/* looks up names in DNS */
 err_t ttoaddr(const char *src, size_t srclen, int af, ip_address *dst);
-
-/* does not look up names in DNS */
-err_t ttoaddr_num(const char *src, size_t srclen, int af, ip_address *dst);
-
 err_t tnatoaddr(const char *src, size_t srclen, int af, ip_address *dst);
 size_t addrtot(const ip_address *src, int format, char *buf, size_t buflen);
 /* RFC 1886 old IPv6 reverse-lookup format is the bulkiest */
@@ -516,8 +511,7 @@ enum klips_debug_flags {
     KDF_RCV         = 9,
     KDF_TUNNEL      = 10,
     KDF_PFKEY       = 11,
-    KDF_COMP        = 12,
-    KDF_NATT        = 13,
+    KDF_COMP        = 12
 };
 
 
