@@ -109,6 +109,36 @@ struct ipsec_birth_reply ipsec_ipv6_birth_packet;
 #ifdef CONFIG_KLIPS_DEBUG
 int debug_esp = 0;
 int debug_ah = 0;
+int sysctl_ipsec_inbound_policy_check = 1;
+int debug_tunnel = 0;
+int debug_xform = 0;
+int debug_eroute = 0;
+int debug_spi = 0;
+int debug_radij = 0;
+int debug_pfkey = 0;
+int debug_rcv = 0;
+int debug_netlink = 0;
+int sysctl_ipsec_debug_verbose = 0;
+int sysctl_ipsec_debug_ipcomp =0;
+int sysctl_ipsec_icmp = 0;
+int sysctl_ipsec_tos = 0;
+#ifdef module_param
+module_param(debug_esp, int, 0644);
+module_param(debug_ah, int, 0644);
+module_param(debug_tunnel, int, 0644);
+module_param(debug_xform, int, 0644);
+module_param(debug_eroute, int, 0644);
+module_param(debug_spi, int, 0644);
+module_param(debug_radij, int, 0644);
+module_param(debug_pfkey, int, 0644);
+module_param(debug_rcv, int, 0644);
+module_param(debug_netlink, int, 0644);
+module_param(sysctl_ipsec_debug_verbose,int, 0644);
+module_param(sysctl_ipsec_debug_ipcomp, int, 0644);
+module_param(sysctl_ipsec_icmp,   int, 0644);
+module_param(sysctl_ipsec_tos,    int, 0644);
+#endif
+
 #endif /* CONFIG_KLIPS_DEBUG */
 
 #define DECREMENT_UNSIGNED(X, amount) ((amount < (X)) ? (X)-amount : 0)
@@ -222,6 +252,16 @@ ipsec_spi_get_info(char *buffer,
 				len += ipsec_snprintf(buffer+len, length-len, " iv_bits=%dbits iv=0x",
 					       sa_p->ips_iv_bits);
 
+#ifdef CONFIG_KLIPS_OCF
+				if (!sa_p->ips_iv) {
+					/* ocf doesn't set the IV, fake it for the UML tests */
+					len += ipsec_snprintf(buffer+len, length-len, "0cf0");
+					for (j = 0; j < (sa_p->ips_iv_bits / 8) - 2; j++) {
+						len += ipsec_snprintf(buffer+len, length-len, "%02x",
+								   (((__u32)sa_p) >> j) & 0xff);
+					}
+				} else
+#endif
 				for(j = 0; j < sa_p->ips_iv_bits / 8; j++) {
 					len += ipsec_snprintf(buffer+len, length-len, "%02x",
 						       (__u32)((__u8*)(sa_p->ips_iv))[j]);
@@ -610,6 +650,13 @@ ipsec_version_get_info(char *buffer,
 	return len;
 }
 
+#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+unsigned int natt_available = 1;
+#else
+unsigned int natt_available = 0;
+#endif
+module_param(natt_available, int, 0444);
+
 IPSEC_PROCFS_DEBUG_NO_STATIC
 int
 ipsec_natt_get_info(char *buffer,
@@ -622,12 +669,7 @@ ipsec_natt_get_info(char *buffer,
 
 	len += ipsec_snprintf(buffer + len,
 			      length-len, "%d\n",
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-			      1
-#else
-			      0
-#endif
-		);
+			      natt_available);
 
 	*start = buffer + (offset - begin);	/* Start of wanted data */
 	len -= (offset - begin);			/* Start slop */
@@ -1007,164 +1049,6 @@ ipsec_proc_cleanup()
 }
 
 /*
- * $Log: ipsec_proc.c,v $
- * Revision 1.41  2005/11/11 04:04:03  paul
- * Fix for compiling without CONFIG_KLIPS_ALG by Toby
- *
- * Revision 1.40  2005/08/26 20:02:24  mcr
- * 	added /proc/net/ipsec/natt file to indicate if NAT-T was compiled
- * 	into KLIPS.
- *
- * Revision 1.39  2005/05/20 03:19:18  mcr
- * 	modifications for use on 2.4.30 kernel, with backported
- * 	printk_ratelimit(). all warnings removed.
- *
- * Revision 1.38  2005/04/29 05:10:22  mcr
- * 	removed from extraenous includes to make unit testing easier.
- *
- * Revision 1.37  2005/04/13 22:49:49  mcr
- * 	moved KLIPS specific snprintf() wrapper to seperate file.
- *
- * Revision 1.36  2005/04/06 17:44:36  mcr
- * 	when NAT-T is compiled out, show encap as "NA"
- *
- * Revision 1.35  2005/01/26 00:50:35  mcr
- * 	adjustment of confusion of CONFIG_IPSEC_NAT vs CONFIG_KLIPS_NAT,
- * 	and make sure that NAT_TRAVERSAL is set as well to match
- * 	userspace compiles of code.
- *
- * Revision 1.34  2004/12/03 21:25:57  mcr
- * 	compile time fixes for running on 2.6.
- * 	still experimental.
- *
- * Revision 1.33  2004/08/17 03:27:23  mcr
- * 	klips 2.6 edits.
- *
- * Revision 1.32  2004/08/03 18:19:08  mcr
- * 	in 2.6, use "net_device" instead of #define device->net_device.
- * 	this probably breaks 2.0 compiles.
- *
- * Revision 1.31  2004/07/10 19:11:18  mcr
- * 	CONFIG_IPSEC -> CONFIG_KLIPS.
- *
- * Revision 1.30  2004/04/25 21:23:11  ken
- * Pull in dhr's changes from FreeS/WAN 2.06
- *
- * Revision 1.29  2004/04/06 02:49:26  mcr
- * 	pullup of algo code from alg-branch.
- *
- * Revision 1.28  2004/03/28 20:29:58  paul
- *      <hugh_> ssize_t, not ssized_t
- *
- * Revision 1.27  2004/03/28 20:27:20  paul
- * Included tested and confirmed fixes mcr made and dhr verified for
- * snprint statements. Changed one other snprintf to use ipsec_snprintf
- * so it wouldnt break compatibility with 2.0/2.2 kernels. Verified with
- * dhr. (thanks dhr!)
- *
- * Revision 1.26  2004/02/09 22:07:06  mcr
- * 	added information about nat-traversal setting to spi-output.
- *
- * Revision 1.25.4.1  2004/04/05 04:30:46  mcr
- * 	patches for alg-branch to compile/work with 2.x openswan
- *
- * Revision 1.25  2003/10/31 02:27:55  mcr
- * 	pulled up port-selector patches and sa_id elimination.
- *
- * Revision 1.24.4.1  2003/10/29 01:30:41  mcr
- * 	elimited "struct sa_id".
- *
- * Revision 1.24  2003/06/20 01:42:21  mcr
- * 	added counters to measure how many ACQUIREs we send to pluto,
- * 	and how many are successfully sent.
- *
- * Revision 1.23  2003/04/03 17:38:09  rgb
- * Centralised ipsec_kfree_skb and ipsec_dev_{get,put}.
- *
- * Revision 1.22  2002/09/20 15:40:57  rgb
- * Renamed saref macros for consistency and brevity.
- *
- * Revision 1.21  2002/09/20 05:01:35  rgb
- * Print ref and  reftable, refentry seperately.
- *
- * Revision 1.20  2002/09/19 02:35:39  mcr
- * 	do not define structures needed by /proc/net/ipsec/ if we
- * 	aren't going create that directory.
- *
- * Revision 1.19  2002/09/10 01:43:25  mcr
- * 	fixed problem in /-* comment.
- *
- * Revision 1.18  2002/09/03 16:22:11  mcr
- * 	fixed initialization of birth/stuff values - some simple
- * 	screw ups in the code.
- * 	removed debugging that was left in by mistake.
- *
- * Revision 1.17  2002/09/02 17:54:53  mcr
- * 	changed how the table driven /proc entries are created so that
- * 	making subdirs is now explicit rather than implicit.
- *
- * Revision 1.16  2002/08/30 01:23:37  mcr
- * 	reorganized /proc creating code to clear up ifdefs,
- * 	make the 2.4 code table driven, and put things into
- * 	/proc/net/ipsec subdir. Symlinks are left for compatibility.
- *
- * Revision 1.15  2002/08/13 19:01:25  mcr
- * 	patches from kenb to permit compilation of FreeSWAN on ia64.
- * 	des library patched to use proper DES_LONG type for ia64.
- *
- * Revision 1.14  2002/07/26 08:48:31  rgb
- * Added SA ref table code.
- *
- * Revision 1.13  2002/07/24 18:44:54  rgb
- * Type fiddling to tame ia64 compiler.
- *
- * Revision 1.12  2002/05/27 18:56:07  rgb
- * Convert to dynamic ipsec device allocation.
- *
- * Revision 1.11  2002/05/23 07:14:50  rgb
- * Added refcount code.
- * Cleaned up %p variants to 0p%p for test suite cleanup.
- * Convert "usecount" to "refcount" to remove ambiguity.
- *
- * Revision 1.10  2002/04/24 07:55:32  mcr
- * 	#include patches and Makefiles for post-reorg compilation.
- *
- * Revision 1.9  2002/04/24 07:36:28  mcr
- * Moved from ./klips/net/ipsec/ipsec_proc.c,v
- *
- * Revision 1.8  2002/01/29 17:17:55  mcr
- * 	moved include of ipsec_param.h to after include of linux/kernel.h
- * 	otherwise, it seems that some option that is set in ipsec_param.h
- * 	screws up something subtle in the include path to kernel.h, and
- * 	it complains on the snprintf() prototype.
- *
- * Revision 1.7  2002/01/29 04:00:52  mcr
- * 	more excise of kversions.h header.
- *
- * Revision 1.6  2002/01/29 02:13:17  mcr
- * 	introduction of ipsec_kversion.h means that include of
- * 	ipsec_param.h must preceed any decisions about what files to
- * 	include to deal with differences in kernel source.
- *
- * Revision 1.5  2002/01/12 02:54:30  mcr
- * 	beginnings of /proc/net/ipsec dir.
- *
- * Revision 1.4  2001/12/11 02:21:05  rgb
- * Don't include module version here, fixing 2.2 compile bug.
- *
- * Revision 1.3  2001/12/05 07:19:44  rgb
- * Fixed extraneous #include "version.c" bug causing modular KLIPS failure.
- *
- * Revision 1.2  2001/11/26 09:16:14  rgb
- * Merge MCR's ipsec_sa, eroute, proc and struct lifetime changes.
- *
- * Revision 1.74  2001/11/22 05:44:11  henry
- * new version stuff
- *
- * Revision 1.1.2.1  2001/09/25 02:19:40  mcr
- * 	/proc manipulation code moved to new ipsec_proc.c
- *
- *
  * Local variables:
  * c-file-style: "linux"
  * End:
