@@ -3,6 +3,10 @@
  * Copyright (C) 1996, 1997  John Ioannidis.
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003  Richard Guy Briggs.
  * 
+ * OCF/receive state machine written by
+ * David McCullough <dmccullough@cyberguard.com>
+ * Copyright (C) 2004-2005 Intel Corporation.  All Rights Reserved.
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
@@ -65,6 +69,8 @@ char ipsec_tunnel_c_version[] = "RCSID $Id: ipsec_tunnel.c,v 1.234 2005/11/11 04
 #include <linux/if_arp.h>
 #include <net/arp.h>
 
+#include <linux/moduleparam.h>
+
 #include "openswan/ipsec_kversion.h"
 #include "openswan/radij.h"
 #include "openswan/ipsec_life.h"
@@ -90,10 +96,6 @@ char ipsec_tunnel_c_version[] = "RCSID $Id: ipsec_tunnel.c,v 1.234 2005/11/11 04
 
 static __u32 zeroes[64];
 
-#ifdef CONFIG_KLIPS_DEBUG
-int debug_tunnel = 0;
-#endif /* CONFIG_KLIPS_DEBUG */
-
 DEBUG_NO_STATIC int
 ipsec_tunnel_open(struct net_device *dev)
 {
@@ -122,15 +124,16 @@ ipsec_tunnel_close(struct net_device *dev)
 	return 0;
 }
 
+#ifdef NETDEV_23
 static inline int ipsec_tunnel_xmit2(struct sk_buff *skb)
 {
-
 #ifdef NETDEV_25	/* 2.6 kernels */
 	return dst_output(skb);
 #else
 	return ip_send(skb);
 #endif
 }
+#endif /* NETDEV_23 */
 
 enum ipsec_xmit_value
 ipsec_tunnel_strip_hard_header(struct ipsec_xmit_state *ixs)
@@ -235,8 +238,8 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 		if(ixs->skb->sk) {
 #ifdef NET_26
 #ifdef HAVE_INET_SK_SPORT
-                      ixs->sport = ntohs(inet_sk(ixs->skb->sk)->sport);
-                      ixs->dport = ntohs(inet_sk(ixs->skb->sk)->dport);
+			ixs->sport = ntohs(inet_sk(ixs->skb->sk)->sport);
+			ixs->dport = ntohs(inet_sk(ixs->skb->sk)->dport);
 #else
 			struct udp_sock *us;
 			
@@ -280,8 +283,8 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 		if(ixs->skb->sk) {
 #ifdef NET_26
 #ifdef HAVE_INET_SK_SPORT
-                       ixs->sport = ntohs(inet_sk(ixs->skb->sk)->sport);
-                       ixs->dport = ntohs(inet_sk(ixs->skb->sk)->dport);
+			ixs->sport = ntohs(inet_sk(ixs->skb->sk)->sport);
+			ixs->dport = ntohs(inet_sk(ixs->skb->sk)->dport);
 #else
 			struct tcp_tw_bucket *tw;
 			
@@ -563,14 +566,15 @@ ipsec_tunnel_restore_hard_header(struct ipsec_xmit_state*ixs)
 enum ipsec_xmit_value
 ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
 {
-	int err;
 #ifdef NETDEV_25
 	struct flowi fl;
 #endif
   
+#ifdef NET_21	/* 2.2 and 2.4 kernels */
 	/* new route/dst cache code from James Morris */
 	ixs->skb->dev = ixs->physdev;
 #ifdef NETDEV_25
+	memset (&fl, 0x0, sizeof (struct flowi));
  	fl.oif = ixs->physdev->iflink;
  	fl.nl_u.ip4_u.daddr = ixs->skb->nh.iph->daddr;
  	fl.nl_u.ip4_u.saddr = ixs->pass ? 0 : ixs->skb->nh.iph->saddr;
@@ -594,7 +598,6 @@ ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
 			    ixs->route->u.dst.dev->name);
 		return IPSEC_XMIT_ROUTEERR;
 	}
-
 	if(ixs->dev == ixs->route->u.dst.dev) {
 		ip_rt_put(ixs->route);
 		/* This is recursion, drop it. */
@@ -607,7 +610,6 @@ ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
 	}
 	dst_release(ixs->skb->dst);
 	ixs->skb->dst = &ixs->route->u.dst;
-
 	ixs->stats->tx_bytes += ixs->skb->len;
 	if(ixs->skb->len < ixs->skb->nh.raw - ixs->skb->data) {
 		ixs->stats->tx_errors++;
@@ -621,8 +623,8 @@ ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
 	__skb_pull(ixs->skb, ixs->skb->nh.raw - ixs->skb->data);
 #ifdef SKB_RESET_NFCT
 	if(!ixs->pass) {
-		nf_conntrack_put(ixs->skb->nfct);
-		ixs->skb->nfct = NULL;
+	  nf_conntrack_put(ixs->skb->nfct);
+	  ixs->skb->nfct = NULL;
 	}
 #if defined(CONFIG_NETFILTER_DEBUG) && defined(HAVE_SKB_NF_DEBUG)
 	ixs->skb->nf_debug = 0;
@@ -633,26 +635,36 @@ ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
 		    "...done, calling ip_send() on device:%s\n",
 		    ixs->skb->dev ? ixs->skb->dev->name : "NULL");
 	KLIPS_IP_PRINT(debug_tunnel & DB_TN_XMIT, ixs->skb->nh.iph);
+#ifdef NETDEV_23	/* 2.4 kernels */
+	{
+		int err;
 
-	if(ixs->pass) {
-		err = ipsec_tunnel_xmit2(ixs->skb);
-	} else {
-		err = NF_HOOK(PF_INET, NF_IP_LOCAL_OUT,
-			      ixs->skb, NULL, ixs->route->u.dst.dev,
-		      ipsec_tunnel_xmit2);
+		err = NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, ixs->skb, NULL, ixs->route->u.dst.dev,
+			      ipsec_tunnel_xmit2);
+		if(err != NET_XMIT_SUCCESS && err != NET_XMIT_CN) {
+			if(net_ratelimit())
+				printk(KERN_ERR
+				       "klips_error:ipsec_xmit_send: "
+				       "ip_send() failed, err=%d\n", 
+				       -err);
+			ixs->stats->tx_errors++;
+			ixs->stats->tx_aborted_errors++;
+			ixs->skb = NULL;
+			return IPSEC_XMIT_IPSENDFAILURE;
+		}
 	}
-	if(err != NET_XMIT_SUCCESS && err != NET_XMIT_CN) {
-		if(net_ratelimit())
-			printk(KERN_ERR
-			       "klips_error:ipsec_xmit_send: "
-			       "ip_send() failed, err=%d\n", 
-			       -err);
-		ixs->stats->tx_errors++;
-		ixs->stats->tx_aborted_errors++;
-		ixs->skb = NULL;
-		return IPSEC_XMIT_IPSENDFAILURE;
-	}
-
+#else /* NETDEV_23 */	/* 2.2 kernels */
+	ip_send(ixs->skb);
+#endif /* NETDEV_23 */
+#else /* NET_21 */	/* 2.0 kernels */
+	ixs->skb->arp = 1;
+	/* ISDN/ASYNC PPP from Matjaz Godec. */
+	/*	skb->protocol = htons(ETH_P_IP); */
+	KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+		    "klips_debug:ipsec_xmit_send: "
+		    "...done, calling dev_queue_xmit() or ip_fragment().\n");
+	IP_SEND(ixs->skb, ixs->physdev);
+#endif /* NET_21 */
 	ixs->stats->tx_packets++;
 
 	ixs->skb = NULL;
@@ -660,6 +672,10 @@ ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
 	return IPSEC_XMIT_OK;
 }
 
+/* management of buffers */
+static struct ipsec_xmit_state * ipsec_xmit_state_new (void);
+static void ipsec_xmit_state_delete (struct ipsec_xmit_state *ixs);
+
 void
 ipsec_tunnel_cleanup(struct ipsec_xmit_state*ixs)
 {
@@ -683,12 +699,71 @@ ipsec_tunnel_cleanup(struct ipsec_xmit_state*ixs)
 	if (ixs->ips.ips_ident_d.data) {
 		kfree(ixs->ips.ips_ident_d.data);
 	}
+        ipsec_xmit_state_delete (ixs);
 }
 
-/* management of buffers */
-static struct ipsec_xmit_state * ipsec_xmit_state_new (void);
-static void ipsec_xmit_state_delete (struct ipsec_xmit_state *ixs);
+/*
+ * when encap processing is complete it call this for us to continue
+ */
 
+void
+ipsec_tunnel_xsm_complete(
+	struct ipsec_xmit_state *ixs,
+	enum ipsec_xmit_value stat)
+{
+	if(stat != IPSEC_XMIT_OK) {
+		if(stat == IPSEC_XMIT_PASS) {
+			goto bypass;
+		}
+		
+		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+				"klips_debug:ipsec_tunnel_start_xmit: encap_bundle failed: %d\n",
+				stat);
+		goto cleanup;
+	}
+
+	ixs->matcher.sen_ip_src.s_addr = ixs->iph->saddr;
+	ixs->matcher.sen_ip_dst.s_addr = ixs->iph->daddr;
+	ixs->matcher.sen_proto = ixs->iph->protocol;
+	ipsec_extract_ports(ixs->iph, &ixs->matcher);
+
+	spin_lock(&eroute_lock);
+	ixs->eroute = ipsec_findroute(&ixs->matcher);
+	if(ixs->eroute) {
+		ixs->outgoing_said = ixs->eroute->er_said;
+		ixs->eroute_pid = ixs->eroute->er_pid;
+		ixs->eroute->er_count++;
+		ixs->eroute->er_lasttime = jiffies/HZ;
+	}
+	spin_unlock(&eroute_lock);
+
+	KLIPS_PRINT((debug_tunnel & DB_TN_XMIT) &&
+			/* ((ixs->orgdst != ixs->newdst) || (ixs->orgsrc != ixs->newsrc)) */
+			(ixs->orgedst != ixs->outgoing_said.dst.u.v4.sin_addr.s_addr) &&
+			ixs->outgoing_said.dst.u.v4.sin_addr.s_addr &&
+			ixs->eroute,
+			"klips_debug:ipsec_tunnel_start_xmit: "
+			"We are recursing here.\n");
+
+	if (/*((ixs->orgdst != ixs->newdst) || (ixs->orgsrc != ixs->newsrc))*/
+			(ixs->orgedst != ixs->outgoing_said.dst.u.v4.sin_addr.s_addr) &&
+			ixs->outgoing_said.dst.u.v4.sin_addr.s_addr &&
+			ixs->eroute) {
+		ipsec_xsm(ixs);
+		return;
+	}
+
+	stat = ipsec_tunnel_restore_hard_header(ixs);
+	if(stat != IPSEC_XMIT_OK) {
+		goto cleanup;
+	}
+
+bypass:
+	stat = ipsec_tunnel_send(ixs);
+
+cleanup:
+	ipsec_tunnel_cleanup(ixs);
+}
 
 /*
  *	This function assumes it is being called from dev_queue_xmit()
@@ -733,60 +808,15 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	}
 	
 	ixs->innersrc = ixs->iph->saddr;
-	/* start encapsulation loop here XXX */
-	do {
- 		stat = ipsec_xmit_encap_bundle(ixs);
-	 	if(stat != IPSEC_XMIT_OK) {
-			if(stat == IPSEC_XMIT_PASS) {
-				goto bypass;
-			}
-			
-			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-				    "klips_debug:ipsec_tunnel_start_xmit: encap_bundle failed: %d\n",
-				    stat);
- 			goto cleanup;
-	 	}
 
-		ixs->matcher.sen_ip_src.s_addr = ixs->iph->saddr;
-		ixs->matcher.sen_ip_dst.s_addr = ixs->iph->daddr;
-		ixs->matcher.sen_proto = ixs->iph->protocol;
-		ipsec_extract_ports(ixs->iph, &ixs->matcher);
+	ixs->xsm_complete = ipsec_tunnel_xsm_complete;
 
-		spin_lock(&eroute_lock);
-		ixs->eroute = ipsec_findroute(&ixs->matcher);
-		if(ixs->eroute) {
-			ixs->outgoing_said = ixs->eroute->er_said;
-			ixs->eroute_pid = ixs->eroute->er_pid;
-			ixs->eroute->er_count++;
-			ixs->eroute->er_lasttime = jiffies/HZ;
-		}
-		spin_unlock(&eroute_lock);
-
-		KLIPS_PRINT((debug_tunnel & DB_TN_XMIT) &&
-			    /* ((ixs->orgdst != ixs->newdst) || (ixs->orgsrc != ixs->newsrc)) */
-			    (ixs->orgedst != ixs->outgoing_said.dst.u.v4.sin_addr.s_addr) &&
-			    ixs->outgoing_said.dst.u.v4.sin_addr.s_addr &&
-			    ixs->eroute,
-			    "klips_debug:ipsec_tunnel_start_xmit: "
-			    "We are recursing here.\n");
-
-	} while(/*((ixs->orgdst != ixs->newdst) || (ixs->orgsrc != ixs->newsrc))*/
-		(ixs->orgedst != ixs->outgoing_said.dst.u.v4.sin_addr.s_addr) &&
-		ixs->outgoing_said.dst.u.v4.sin_addr.s_addr &&
-		ixs->eroute);
-	
-	stat = ipsec_tunnel_restore_hard_header(ixs);
-	if(stat != IPSEC_XMIT_OK) {
-		goto cleanup;
-	}
-
- bypass:
-	stat = ipsec_tunnel_send(ixs);
+	ipsec_xsm(ixs);
+	return 0;
 
  cleanup:
 	ipsec_tunnel_cleanup(ixs);
 
-        ipsec_xmit_state_delete (ixs);
 alloc_error:
 	return 0;
 }
@@ -1849,6 +1879,10 @@ static spinlock_t ixs_cache_lock = SPIN_LOCK_UNLOCKED;
 static kmem_cache_t *ixs_cache_allocator = NULL;
 static unsigned  ixs_cache_allocated_count = 0;
 
+static int ipsec_ixs_max = 1000;
+module_param(ipsec_ixs_max,int,0644);
+MODULE_PARM_DESC(ipsec_ixs_max, "Maximum outstanding transmit packets");
+
 int
 ipsec_xmit_state_cache_init (void)
 {
@@ -1882,22 +1916,53 @@ ipsec_xmit_state_cache_cleanup (void)
 static struct ipsec_xmit_state *
 ipsec_xmit_state_new (void)
 {
-	struct ipsec_xmit_state *ixs;
+	struct ipsec_xmit_state *ixs = NULL;
 
         spin_lock_bh (&ixs_cache_lock);
 
-        ixs = kmem_cache_alloc (ixs_cache_allocator, GFP_ATOMIC);
-
-        if (likely (ixs != NULL))
-                ixs_cache_allocated_count++;
+        if (likely (ixs_cache_allocated_count >= ipsec_ixs_max)) {
+		KLIPS_PRINT(debug_tunnel,
+			    "klips_debug:ipsec_xmit_state_new: "
+			    "exceeded outstanding TX packet cnt %d\n", ipsec_ixs_max);
+	} else {
+                ixs = kmem_cache_alloc (ixs_cache_allocator, GFP_ATOMIC);
+                if (likely (ixs != NULL))
+                        ixs_cache_allocated_count++;
+        }
 
         spin_unlock_bh (&ixs_cache_lock);
 
         if (unlikely (NULL == ixs))
                 goto bail;
 
-        // initialize the object
-        memset((caddr_t)ixs, 0, sizeof(*ixs));
+#if 0 /* optimised to only clear the required bits */
+	memset((caddr_t)ixs, 0, sizeof(*ixs));
+#else
+	ixs->pass = 0;
+	ixs->state = 0;
+	ixs->next_state = 0;
+	ixs->ipsp = NULL;
+	ixs->ipsq = NULL;
+	ixs->sa_len = 0;
+	ixs->stats = NULL;
+	ixs->ips.ips_ident_s.data = NULL;
+	ixs->ips.ips_ident_d.data = NULL;
+	ixs->outgoing_said.proto = 0;
+#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+	ixs->natt_type = 0, ixs->natt_head = 0;
+	ixs->natt_sport = 0, ixs->natt_dport = 0;
+#endif
+	ixs->tot_headroom = 0;
+	ixs->tot_tailroom = 0;
+	ixs->eroute = NULL;
+	ixs->hard_header_stripped = 0;
+	ixs->hard_header_len = 0;
+	ixs->cur_mtu = 0; /* FIXME: can we do something better ? */
+
+	ixs->oskb = NULL;
+	ixs->saved_header = NULL;	/* saved copy of the hard header */
+	ixs->route = NULL;
+#endif /* memset */
 
 bail:
         return ixs;
