@@ -1,6 +1,6 @@
 /* 
  * Cryptographic helper function - calculate DH
- * Copyright (C) 2007 Michael C. Richardson <mcr@xelerance.com>
+ * Copyright (C) 2004 Michael C. Richardson <mcr@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,6 +14,7 @@
  *
  * This code was developed with the support of IXIA communications.
  *
+ * RCSID $Id: crypt_dh.c,v 1.11 2005/08/14 21:47:29 mcr Exp $
  */
 
 #include <stdlib.h>
@@ -48,25 +49,53 @@
 #include "id.h"
 #include "secrets.h"
 #include "keys.h"
-#include "ikev2_prfplus.h"
+
+#ifdef VULCAN_PK
+#include "dev/hifn/vulcanpk_funcs.h"
+#endif
 
 /** Compute DH shared secret from our local secret and the peer's public value.
  * We make the leap that the length should be that of the group
  * (see quoted passage at start of ACCEPT_KE).
  */
 static void
-calc_dh_shared(chunk_t *shared, const chunk_t g
-	       , const MP_INT *sec
-	       , const struct oakley_group_desc *group)
+calc_dh_shared_gmp(chunk_t *shared, const chunk_t g
+		   , const chunk_t *secchunk
+		   , const struct oakley_group_desc *group)
 {
-    MP_INT mp_g, mp_shared;
+    MP_INT mp_g, mp_shared, sec;
     struct timeval tv0, tv1;
     unsigned long tv_diff;
 
     gettimeofday(&tv0, NULL);
+
+    /* recover the long term secret */
+    n_to_mpz(&sec, secchunk->ptr, secchunk->len);
+
+    /* do the exponentiation */
     n_to_mpz(&mp_g, g.ptr, g.len);
     mpz_init(&mp_shared);
-    mpz_powm(&mp_shared, &mp_g, sec, group->modulus);
+
+#ifdef PK_DH_REGRESS
+    if(pkdh_verbose) {
+	printf("mp_g: ");
+	mpz_out_str(stdout, 16, &mp_g);
+	printf("\nsecret: ");
+	mpz_out_str(stdout, 16, &sec);
+	printf("\nmodulus: ");
+	mpz_out_str(stdout, 16, group->modulus);
+    }
+#endif
+
+    mpz_powm(&mp_shared, &mp_g, &sec, group->modulus);
+
+#ifdef PK_DH_REGRESS
+    if(pkdh_verbose) {
+	printf("\nshared: ");
+	mpz_out_str(stdout, 16, &mp_shared);
+	printf("\n");
+    }
+#endif
     mpz_clear(&mp_g);
 
     *shared = mpz_to_n(&mp_shared, group->bytes);
@@ -90,6 +119,11 @@ calc_dh_shared(chunk_t *shared, const chunk_t g
     DBG_cond_dump_chunk(DBG_CRYPT, "DH shared-secret:\n", *shared);
 }
 
+#ifdef VULCAN_PK
+#include "vulcan/crypto_dh_vulcanpk.c"
+#endif
+
+calc_dh_shared_t calc_dh_shared = calc_dh_shared_gmp;
 
 /* SKEYID for preshared keys.
  * See draft-ietf-ipsec-ike-01.txt 4.1
@@ -166,7 +200,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
     )
 {
     oakley_auth_t auth = skq->auth;
-    oakley_hash_t hash = skq->prf_hash;
+    oakley_hash_t hash = skq->hash;
     const struct hash_desc *hasher = crypto_get_hasher(hash);
     chunk_t pss;  
     chunk_t ni;
@@ -296,14 +330,15 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
 	DBG_dump_chunk("IV:",       *new_iv));
 }
 
+
 void calc_dh_iv(struct pluto_crypto_req *r)
 {
     struct pcr_skeyid_q *skq = &r->pcr_d.dhq;
     struct pcr_skeyid_r *skr = &r->pcr_d.dhr;
     struct pcr_skeyid_q dhq;
     const struct oakley_group_desc *group;
-    MP_INT  sec;
-    chunk_t  shared, g, ltsecret;
+    chunk_t  sec;
+    chunk_t  shared, g;
     chunk_t  skeyid, skeyid_d, skeyid_a, skeyid_e; 
     chunk_t  new_iv, enc_key;
 
@@ -324,26 +359,21 @@ void calc_dh_iv(struct pluto_crypto_req *r)
     shared.ptr = wire_chunk_ptr(skr, &skr->shared);
     shared.len = group->bytes;
 
-    ltsecret.ptr = wire_chunk_ptr(&dhq, &dhq.secret);
-    ltsecret.len = dhq.secret.len;
-
-    /* recover the long term secret */
-    n_to_mpz(&sec, ltsecret.ptr, ltsecret.len);
-
-    /* now calculate the (g^x)(g^y) ---
-       need gi on responder, gr on initiator */
-
-    if(dhq.init == RESPONDER) {
-	setchunk_fromwire(g, &dhq.gi, &dhq);
-    } else {
-	setchunk_fromwire(g, &dhq.gr, &dhq);
-    }
+    sec.ptr = wire_chunk_ptr(&dhq, &dhq.secret);
+    sec.len = dhq.secret.len;
 
     DBG(DBG_CRYPT,
-	DBG_dump_chunk("peer's g: ", g);
-	DBG_dump_chunk("long term secret: ", ltsecret));
+	DBG_dump_chunk("long term secret: ", sec));
 
-    calc_dh_shared(&shared, g, &sec, group);
+    /* now calculate the (g^x)(g^y) --- need gi on responder, gr on initiator */
+
+    if(dhq.init == RESPONDER) {
+      setchunk_fromwire(g, &dhq.gi, &dhq);
+    } else {
+      setchunk_fromwire(g, &dhq.gr, &dhq);
+    }
+
+    (*calc_dh_shared)(&shared, g, &sec, group);
     
     memset(&skeyid, 0, sizeof(skeyid));
     memset(&skeyid_d, 0, sizeof(skeyid_d));
@@ -351,7 +381,6 @@ void calc_dh_iv(struct pluto_crypto_req *r)
     memset(&skeyid_e, 0, sizeof(skeyid_e));
     memset(&new_iv,   0, sizeof(new_iv));
     memset(&enc_key,  0, sizeof(enc_key));
-
     /* okay, so now calculate IV */
     calc_skeyids_iv(&dhq
 		    , shared
@@ -389,7 +418,7 @@ void calc_dh(struct pluto_crypto_req *r)
     struct pcr_skeyid_r *skr = &r->pcr_d.dhr;
     struct pcr_skeyid_q dhq;
     const struct oakley_group_desc *group;
-    MP_INT  sec;
+    chunk_t  sec;
     chunk_t  shared, g;
 
     /* copy the request, since we will use the same memory for the reply */
@@ -409,7 +438,8 @@ void calc_dh(struct pluto_crypto_req *r)
     shared.len = group->bytes;
 
     /* recover the long term secret */
-    n_to_mpz(&sec, wire_chunk_ptr(&dhq, &dhq.secret), dhq.secret.len);
+    sec.ptr = wire_chunk_ptr(&dhq, &dhq.secret);
+    sec.len = dhq.secret.len;
 
     /* now calculate the (g^x)(g^y) */
 
@@ -418,7 +448,7 @@ void calc_dh(struct pluto_crypto_req *r)
     } else {
       setchunk_fromwire(g, &dhq.gr, &dhq);
     }
-    calc_dh_shared(&shared, g, &sec, group);
+    (*calc_dh_shared)(&shared, g, &sec, group);
 
     /* now translate it back to wire chunks, freeing the chunks */
     setwirechunk_fromchunk(skr->shared,   shared,   skr);
@@ -427,214 +457,6 @@ void calc_dh(struct pluto_crypto_req *r)
     return;
 }
 
-/* 
- * IKEv2 - RFC4306 SKEYSEED - calculation.
- */
-
-static void
-calc_skeyseed_v2(struct pcr_skeyid_q *skq
-		 , chunk_t shared
-		 , const size_t keysize
-		 , chunk_t *skeyseed
-		 , chunk_t *SK_d
-		 , chunk_t *SK_ai
-		 , chunk_t *SK_ar
-		 , chunk_t *SK_ei
-		 , chunk_t *SK_er
-		 , chunk_t *SK_pi
-		 , chunk_t *SK_pr
-    )
-{
-    struct v2prf_stuff vpss;
-    chunk_t gi, gr;
-    memset(&vpss, 0, sizeof(vpss));
-
-    /* this doesn't take any memory, it's just moving pointers around */
-    setchunk_fromwire(gi,      &skq->gi, skq);
-    setchunk_fromwire(gr,      &skq->gr, skq);
-    setchunk_fromwire(vpss.ni, &skq->ni, skq);
-    setchunk_fromwire(vpss.nr, &skq->nr, skq);
-    setchunk_fromwire(vpss.spii, &skq->icookie, skq);
-    setchunk_fromwire(vpss.spir, &skq->rcookie, skq);
-
-    DBG(DBG_CONTROLMORE
-	, DBG_log("calculating skeyseed using prf=%s integ=%s cipherkey=%u"
-		  , enum_name(&trans_type_prf_names, skq->prf_hash)
-		  , enum_name(&trans_type_integ_names, skq->integ_hash)
-		  , keysize));
-
-
-    vpss.prf_hasher = crypto_get_hasher(skq->prf_hash);
-    passert(vpss.prf_hasher);
-
-    /* generate SKEYSEED from key=(Ni|Nr), hash of shared */
-    {
-	struct hmac_ctx ctx;
-	unsigned int keybytes;
-	unsigned char *kb;
-
-	//if(vpss.prf_hasher->hash_key_size == 0) {
-	keybytes = vpss.ni.len + vpss.nr.len;
-	//} else {
-	//keybytes = vpss.prf_hasher->hash_key_size;
-	//}
-
-	kb = alloc_bytes(keybytes, "skeyseed prf key");
-	memset(kb, 0, keybytes);
-	memcpy(kb,               vpss.ni.ptr, vpss.ni.len);
-	memcpy(kb + vpss.ni.len, vpss.nr.ptr, vpss.nr.len);
-
-	/* SKEYSEED */
-	DBG(DBG_CRYPT,
-	    DBG_dump("Input to SKEYSEED: ", kb, keybytes));
-
-	hmac_init(&ctx, vpss.prf_hasher, kb, keybytes);
-	hmac_update_chunk(&ctx, shared);
-	hmac_final_chunk(*skeyseed, "skeyseed base", &ctx);
-	vpss.skeyseed = skeyseed;
-	pfree(kb);
-    }
-
-    /* now we have to generate the keys for everything */
-    {
-	/* need to know how many bits to generate */
-	/* SK_d needs PRF hasher key bits */
-	/* SK_p needs PRF hasher*2 key bits */
-	/* SK_e needs keysize*2 key bits */
-	/* SK_a needs hash's key bits size */
-	const struct hash_desc *integ_hasher = crypto_get_hasher(skq->integ_hash);
-	int skd_bytes = vpss.prf_hasher->hash_key_size;
-	int ska_bytes = integ_hasher->hash_key_size;
-	int ske_bytes = keysize;
-	int skp_bytes = vpss.prf_hasher->hash_key_size;
-
-	vpss.counter[0]=0x01;
-	vpss.t.len = 0;
-
-	if(DBGP(DBG_CRYPT)) {
-	    DBG_log("PRF+ input");
-	    DBG_dump_chunk("Ni", vpss.ni);
-	    DBG_dump_chunk("Nr", vpss.nr);
-	    DBG_dump_chunk("SPIi", vpss.spii);
-	    DBG_dump_chunk("SPIr", vpss.spir);
-	}
-	
-	/* SKEYSEED_T1 */
-	v2genbytes(SK_d,  skd_bytes, "SK_d", &vpss);
-	v2genbytes(SK_ai, ska_bytes, "SK_ai", &vpss);
-	v2genbytes(SK_ar, ska_bytes, "SK_ar", &vpss);
-	v2genbytes(SK_ei, ske_bytes, "SK_ei", &vpss);
-	v2genbytes(SK_er, ske_bytes, "SK_er", &vpss);
-	v2genbytes(SK_pi, skp_bytes, "SK_ei", &vpss);
-	v2genbytes(SK_pr, skp_bytes, "SK_er", &vpss);
-    }
-
-    DBG(DBG_CRYPT,
-	DBG_dump_chunk("shared:  ", shared);
-	DBG_dump_chunk("skeyseed:", *skeyseed);
-	DBG_dump_chunk("SK_d:", *SK_d);
-	DBG_dump_chunk("SK_ai:", *SK_ai);
-	DBG_dump_chunk("SK_ar:", *SK_ar);
-	DBG_dump_chunk("SK_ei:", *SK_ei);
-	DBG_dump_chunk("SK_er:", *SK_er);
-	DBG_dump_chunk("SK_pi:", *SK_pi);
-	DBG_dump_chunk("SK_pr:", *SK_pr));
-}
-
-void calc_dh_v2(struct pluto_crypto_req *r)
-{
-    struct pcr_skeyid_q    *skq = &r->pcr_d.dhq;
-    struct pcr_skeycalc_v2 *skr = &r->pcr_d.dhv2;
-    struct pcr_skeyid_q dhq;
-    const struct oakley_group_desc *group;
-    MP_INT  sec;
-    chunk_t  shared, g, ltsecret;
-    chunk_t  skeyseed;
-    chunk_t  SK_d, SK_ai, SK_ar, SK_ei, SK_er, SK_pi, SK_pr;
-
-    /* copy the request, since we will use the same memory for the reply */
-    memcpy(&dhq, skq, sizeof(struct pcr_skeyid_q));
-
-    /* clear out the reply */
-    memset(skr, 0, sizeof(*skr));
-    skr->thespace.start = 0;
-    skr->thespace.len   = sizeof(skr->space);
-
-    group = lookup_group(dhq.oakley_group);
-    passert(group != NULL);
-
-    pluto_crypto_allocchunk(&skr->thespace
-			   , &skr->shared
-			   , group->bytes);
-    shared.ptr = wire_chunk_ptr(skr, &skr->shared);
-    shared.len = group->bytes;
-
-    ltsecret.ptr = wire_chunk_ptr(&dhq, &dhq.secret);
-    ltsecret.len = dhq.secret.len;
-
-    /* recover the long term secret */
-    n_to_mpz(&sec, ltsecret.ptr, ltsecret.len);
-
-    DBG(DBG_CRYPT,
-	DBG_dump_chunk("long term secret: ", ltsecret));
-
-    /* now calculate the (g^x)(g^y) --- need gi on responder, gr on initiator */
-
-    if(dhq.init == RESPONDER) {
-	setchunk_fromwire(g, &dhq.gi, &dhq);
-    } else {
-	setchunk_fromwire(g, &dhq.gr, &dhq);
-    }
-
-    calc_dh_shared(&shared, g, &sec, group);
-    
-    memset(&skeyseed,  0, sizeof(skeyseed));
-    memset(&SK_d,      0, sizeof(SK_d));
-    memset(&SK_ai,     0, sizeof(SK_ai));
-    memset(&SK_ar,     0, sizeof(SK_ar));
-    memset(&SK_ei,     0, sizeof(SK_ei));
-    memset(&SK_er,     0, sizeof(SK_er));
-    memset(&SK_pi,     0, sizeof(SK_pi));
-    memset(&SK_pr,     0, sizeof(SK_pr));
-
-    /* okay, so now calculate IV */
-    calc_skeyseed_v2(&dhq
-		     , shared
-		     , dhq.keysize
-		     , &skeyseed
-		     , &SK_d
-		     , &SK_ai
-		     , &SK_ar
-		     , &SK_ei
-		     , &SK_er
-		     , &SK_pi
-		     , &SK_pr);
-
-
-    /* now translate it back to wire chunks, freeing the chunks */
-    setwirechunk_fromchunk(skr->shared,   shared,   skr);
-    setwirechunk_fromchunk(skr->skeyseed, skeyseed, skr);
-    setwirechunk_fromchunk(skr->skeyid_d, SK_d, skr);
-    setwirechunk_fromchunk(skr->skeyid_ai,SK_ai, skr);
-    setwirechunk_fromchunk(skr->skeyid_ar,SK_ar, skr);
-    setwirechunk_fromchunk(skr->skeyid_ei,SK_ei, skr);
-    setwirechunk_fromchunk(skr->skeyid_er,SK_er, skr);
-    setwirechunk_fromchunk(skr->skeyid_pi,SK_pi, skr);
-    setwirechunk_fromchunk(skr->skeyid_pr,SK_pr, skr);
-
-    freeanychunk(shared);
-    freeanychunk(skeyseed);
-    freeanychunk(SK_d);
-    freeanychunk(SK_ai);
-    freeanychunk(SK_ar);
-    freeanychunk(SK_ei);
-    freeanychunk(SK_er);
-    freeanychunk(SK_pi);
-    freeanychunk(SK_pr);
-
-    return;
-}
-
 
 /*
  * Local Variables:
