@@ -50,13 +50,13 @@
 #include "log.h"
 #include "timer.h"
 
-#ifdef HAVE_OCF
+#ifdef HAVE_OCF_AND_OPENSSL
 #include "id.h"
 #include "pgp.h"
 #include "x509.h"
 #include "certs.h"
 #include "keys.h"
-#include "ocf_pk.h"
+#include "ocf_cryptodev.h"
 #endif
 
 void calc_ke(struct pluto_crypto_req *r)
@@ -66,6 +66,9 @@ void calc_ke(struct pluto_crypto_req *r)
     const struct oakley_group_desc *group;
     chunk_t gi;
     struct pcr_kenonce *kn = &r->pcr_d.kn;
+#ifdef HAVE_OCF_AND_OPENSSL
+    BIGNUM r0;
+#endif
     
     group = lookup_group(kn->oakley_group);
     
@@ -78,7 +81,13 @@ void calc_ke(struct pluto_crypto_req *r)
     n_to_mpz(&secret, wire_chunk_ptr(kn, &(kn->secret)), LOCALSECRETSIZE);
     
     mpz_init(&mp_g);
-    cryptodev.mod_exp(&mp_g, &groupgenerator, &secret, group->modulus);
+#ifdef HAVE_OCF_AND_OPENSSL
+    BN_init(&r0);
+    cryptodev.mod_exp(&r0, &groupgenerator, &secret, group->modulus);
+    bn2mp(&r0, (MP_INT *) &mp_g);
+#else
+    mpz_powm(&mp_g, &groupgenerator, &secret, group->modulus);
+#endif
     
     gi = mpz_to_n(&mp_g, group->bytes);
     
@@ -97,7 +106,11 @@ void calc_ke(struct pluto_crypto_req *r)
 	DBG_dump_chunk("Public DH value sent:\n", gi));
 
     /* clean up after ourselves */
+#ifdef HAVE_OCF_AND_OPENSSL
+    BN_free(&r0);
+#else
     mpz_clear(&mp_g);
+#endif
     freeanychunk(gi);
 }
 
@@ -119,12 +132,13 @@ stf_status build_ke(struct pluto_crypto_req_cont *cn
 		    , const struct oakley_group_desc *group
 		    , enum crypto_importance importance)
 {
-  struct pluto_crypto_req *r;
-  err_t e;
-  bool toomuch = FALSE;
+    struct pluto_crypto_req rd;
+    struct pluto_crypto_req *r = &rd;
+    err_t e;
+    bool toomuch = FALSE;
+
+    memset(&rd, 0, sizeof(rd));
 
-  r = alloc_thing(struct pluto_crypto_req, "build ke request");
-  
   r->pcr_len  = sizeof(struct pluto_crypto_req);
   r->pcr_type = pcr_build_kenonce;
   r->pcr_pcim = importance;
@@ -161,11 +175,12 @@ stf_status build_nonce(struct pluto_crypto_req_cont *cn
 		       , struct state *st 
 		       , enum crypto_importance importance)
 {
-  struct pluto_crypto_req *r;
-  err_t e;
-  bool toomuch = FALSE;
+    struct pluto_crypto_req rd;
+    struct pluto_crypto_req *r = &rd;
+    err_t e;
+    bool toomuch = FALSE;
 
-  r = alloc_thing(struct pluto_crypto_req, "build ke request");
+    memset(&rd, 0, sizeof(rd));
   
   r->pcr_len  = sizeof(struct pluto_crypto_req);
   r->pcr_type = pcr_build_nonce;
