@@ -675,6 +675,7 @@ quick_outI1_continue(struct pluto_crypto_req_cont *pcrc
     passert(st != NULL);
 
     set_cur_state(st);	/* we must reset before exit */
+    set_suspended(st, NULL);
     e = quick_outI1_tail(pcrc, r);
 
     reset_globals();
@@ -745,18 +746,15 @@ quick_outI1(int whack_sock
 	pfsgroupname = enum_name(&oakley_group_names, st->st_pfs_group->group);
     }
 
-    {
-	char replacestr[32];
-
-	replacestr[0]='\0';
-	if(replacing != SOS_NOBODY)
-	    snprintf(replacestr, 32, " to replace #%lu", replacing);
-	
-	openswan_log("initiating Quick Mode %s%s {using isakmp#%lu msgid:%08x proposal=%s pfsgroup=%s}"
+    if (replacing == SOS_NOBODY)
+	openswan_log("initiating Quick Mode %s {using isakmp#%lu proposal=%s pfsgroup=%s}"
 		     , prettypolicy(policy)
-		     , replacestr
-		     , isakmp_sa->st_serialno, st->st_msgid, p2alg, pfsgroupname);
-    }
+		     , isakmp_sa->st_serialno, p2alg, pfsgroupname);
+    else
+	openswan_log("initiating Quick Mode %s to replace #%lu {using isakmp#%lu proposal=%s pfsgroup=%s}"
+		     , prettypolicy(policy)
+		     , replacing
+		     , isakmp_sa->st_serialno, p2alg, pfsgroupname);
 
     qke->st = st;
     qke->isakmp_sa = isakmp_sa;
@@ -997,6 +995,15 @@ quick_outI1_tail(struct pluto_crypto_req_cont *pcrc
  * consulted asynchronously, gateways_from_dns != NULL the second time.
  * Remember that our state object might disappear too!
  *
+ * At the end of authtail, we have all the info we need, but we
+ * haven't done any nonce generation or DH that we might need
+ * to do, so that are two crypto continuations that do this work,
+ * they are:
+ *    quick_inI1_outR1_cryptocontinue1 -- called after NONCE/KE
+ *    quick_inI1_outR1_cryptocontinue2 -- called after DH (if PFS)
+ *
+ * we have to call nonce/ke and DH if we are doing PFS.
+ *
  *
  * If the connection is opportunistic, we must verify delegation.
  *
@@ -1239,7 +1246,7 @@ quick_inI1_outR1_continue(struct adns_continuation *cr, err_t ugh)
     if (st != NULL)
     {
 	passert(st->st_suspended_md == b->md);
-	st->st_suspended_md = NULL;	/* no longer connected or suspended */
+	set_suspended(st, NULL);	/* no longer connected or suspended */
 	cur_state = st;
 	if (!b->failure_ok && ugh != NULL)
 	{
@@ -1276,7 +1283,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
     b->step = next_step;    /* not just vc->b.step */
     vc->b = *b;
     passert(p1st->st_suspended_md == NULL);
-    p1st->st_suspended_md = b->md;
+    set_suspended(p1st, b->md);
 
     DBG(DBG_CONTROL,
 	{
@@ -1368,7 +1375,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
 	 * into b, not just vc->b.
 	 */
 	report_verify_failure(b, ugh);
-	p1st->st_suspended_md = NULL;
+	set_suspended(p1st,  NULL);
 	return STF_FAIL + INVALID_ID_INFORMATION;
     }
     else
@@ -1553,40 +1560,20 @@ quick_inI1_outR1_process_answer(struct verify_oppo_bundle *b
     return next_step;
 }
 
+/* forward definitions */
 static stf_status
-quick_inI1_outR1_cryptotail(struct qke_continuation *qke
+quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 			    , struct pluto_crypto_req *r);
 
 static void
-quick_inI1_outR1_cryptocontinue(struct pluto_crypto_req_cont *pcrc
+quick_inI1_outR1_cryptocontinue2(struct pluto_crypto_req_cont *pcrc
 			      , struct pluto_crypto_req *r
-			      , err_t ugh)
-{
-    struct qke_continuation *qke = (struct qke_continuation *)pcrc;
-    struct state *const st = qke->st;
-    stf_status e;
+				 , err_t ugh);
 
-    DBG(DBG_CONTROLMORE
-	, DBG_log("quick inI1_outR1: calculated ke+nonce, sending R1"));
-
-    /* XXX should check out ugh */
-    passert(ugh == NULL);
-    passert(cur_state == NULL);
-    passert(st != NULL);
-
-    passert(st->st_connection != NULL);
-
-    set_cur_state(st);	/* we must reset before exit */
-    st->st_calculating=FALSE;
-    e = quick_inI1_outR1_cryptotail(qke, r);
-
-    if(qke->md != NULL) {
-	complete_state_transition(&qke->md, e);
-	release_md(qke->md);
-    }
-
-    reset_cur_state();
-}
+static void
+quick_inI1_outR1_cryptocontinue1(struct pluto_crypto_req_cont *pcrc
+				 , struct pluto_crypto_req *r
+				 , err_t ugh);
 
 static stf_status
 quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
@@ -1599,17 +1586,6 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	, *his_net = &b->his.net;
     struct hidden_variables hv;
 
-    {
-	char s1[SUBNETTOT_BUF],d1[SUBNETTOT_BUF];
-    
-	subnettot(our_net, 0, s1, sizeof(s1));
-	subnettot(his_net, 0, d1, sizeof(d1));
-	
-	openswan_log("the peer proposed: %s:%d/%d -> %s:%d/%d"
-		     , s1, c->spd.this.protocol, c->spd.this.port
-		     , d1, c->spd.that.protocol, c->spd.that.port);
-    }
-
     /* Now that we have identities of client subnets, we must look for
      * a suitable connection (our current one only matches for hosts).
      */
@@ -1814,7 +1790,7 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 
 	insert_state(st);	/* needs cookies, connection, and msgid */
 
-	/* copy hidden variables (possibly with changes) */
+	/* copy hidden variables (possibly with changes */
 	st->hidden_variables = hv;
 
 	/* copy the connection's
@@ -1880,7 +1856,7 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	    qke->st = st;
 	    qke->isakmp_sa = p1st;
 	    qke->md = md;
-	    qke->qke_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue;
+	    qke->qke_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue1;
 
 	    if (st->st_pfs_group != NULL) {
 		e = build_ke(&qke->qke_pcrc, st, st->st_pfs_group, ci);
@@ -1895,11 +1871,114 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
     }
 }
 
+static void
+quick_inI1_outR1_cryptocontinue1(struct pluto_crypto_req_cont *pcrc
+				 , struct pluto_crypto_req *r
+				 , err_t ugh)
+{
+    struct qke_continuation *qke = (struct qke_continuation *)pcrc;
+    struct msg_digest *md = qke->md;
+    struct state *const st = qke->st;
+    stf_status e;
+
+    DBG(DBG_CONTROLMORE
+	, DBG_log("quick inI1_outR1: calculated ke+nonce, calculating DH"));
+
+    /* XXX should check out ugh */
+    passert(ugh == NULL);
+    passert(cur_state == NULL);
+    passert(st != NULL);
+
+    passert(st->st_connection != NULL);
+
+    set_cur_state(st);	/* we must reset before exit */
+    st->st_calculating=FALSE;
+    set_suspended(st, NULL);
+
+    /* we always calcualte a nonce */
+    unpack_nonce(&st->st_nr, r);
+
+    if (st->st_pfs_group != NULL) {
+	struct dh_continuation *dh = alloc_thing(struct dh_continuation
+						 , "quick outR1 DH");
+
+	unpack_KE(st, r, &st->st_gr);
+    
+	/* set up second calculation */
+	dh->md = md;
+	set_suspended(st, md);
+	dh->dh_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue2;
+	e = start_dh_secret(&dh->dh_pcrc, st
+			    , st->st_import
+			    , RESPONDER
+			    , st->st_pfs_group->group);
+	
+	if(e != STF_SUSPEND) {
+	    if(dh->md != NULL) {
+		complete_state_transition(&qke->md, e);
+		if(dh->md) release_md(qke->md);
+	    }
+	}
+	
+    } else {
+	/* but if PFS is off, we don't do a second DH, so
+	 * just call the continuation after making something up.
+	 */
+	struct dh_continuation dh;
+
+	dh.md=md;
+
+	e = quick_inI1_outR1_cryptotail(&dh, NULL);
+
+	if(dh.md != NULL) {
+	    /* note: use qke-> pointer */
+	    complete_state_transition(&qke->md, e);
+	    if(dh.md) release_md(qke->md);
+	}
+    }
+    reset_cur_state();
+	
+}
+
+static void
+quick_inI1_outR1_cryptocontinue2(struct pluto_crypto_req_cont *pcrc
+			      , struct pluto_crypto_req *r
+			      , err_t ugh)
+{
+    struct dh_continuation *dh = (struct dh_continuation *)pcrc;
+    struct msg_digest *md = dh->md;
+    struct state *const st = md->st;
+    stf_status e;
+
+    DBG(DBG_CONTROLMORE
+	, DBG_log("quick inI1_outR1: calculated DH, sending R1"));
+
+    /* XXX should check out ugh */
+    passert(ugh == NULL);
+    passert(cur_state == NULL);
+    passert(st != NULL);
+
+    passert(st->st_connection != NULL);
+
+    set_cur_state(st);	/* we must reset before exit */
+    st->st_calculating=FALSE;
+    set_suspended(st, NULL);
+
+    e = quick_inI1_outR1_cryptotail(dh, r);
+
+    if(dh->md != NULL) {
+	complete_state_transition(&dh->md, e);
+	if(dh->md) release_md(dh->md);
+    }
+
+    reset_cur_state();
+}
+
 static stf_status
-quick_inI1_outR1_cryptotail(struct qke_continuation *qke
+quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 			   , struct pluto_crypto_req *r)
 {
-    struct msg_digest *md = qke->md;
+    struct msg_digest *md = dh->md;
     struct state *st = md->st;
     struct connection *c = st->st_connection;
     struct payload_digest *const id_pd = md->chain[ISAKMP_NEXT_ID];
@@ -1954,13 +2033,7 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
     
     {
 	int np;
-#ifdef IMPAIR_UNALIGNED_R1_MSG
-	char *padstr=getenv("PLUTO_UNALIGNED_R1_MSG");
 
-	if(padstr) {
-	    np = ISAKMP_NEXT_VID;
-	} else
-#endif
 	if(st->st_pfs_group != NULL) {
 	    np = ISAKMP_NEXT_KE;
 	} else if(id_pd != NULL) {
@@ -1970,53 +2043,19 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
 	}
 
 	/* Nr out */
-	if (!ship_nonce(&st->st_nr, r, &md->rbody
-			, np, "Nr"))
+	if (!justship_nonce(&st->st_nr, &md->rbody, np, "Nr"))
 	    return STF_INTERNAL_ERROR;
-
-
-#ifdef IMPAIR_UNALIGNED_R1_MSG
-	if(padstr) {
-	    pb_stream vid_pbs;
-	    int padsize;
-	    padsize = strtoul(padstr, NULL, 0);
-	    
-	    openswan_log("inserting fake VID payload of %u size", padsize);
-	    
-	    if(st->st_pfs_group != NULL) {
-		np = ISAKMP_NEXT_KE;
-	    } else if(id_pd != NULL) {
-		np = ISAKMP_NEXT_ID;
-	    } else {
-		np = ISAKMP_NEXT_NONE;
-	    }
-
-	    if (!out_generic(np,
-			     &isakmp_vendor_id_desc, &md->rbody, &vid_pbs))
-		return STF_INTERNAL_ERROR;
-	    
-	    if (!out_zero(padsize, &vid_pbs, "Filler VID"))
-		return STF_INTERNAL_ERROR;
-		
-	    close_output_pbs(&vid_pbs);
-	} 
-#endif
     }
     
-
     /* [ KE ] out (for PFS) */
-    if (st->st_pfs_group != NULL) {
-	stf_status stat;
-
-	if (!ship_KE(st, r, &st->st_gr
-		     , &md->rbody
-		     , id_pd != NULL? ISAKMP_NEXT_ID : ISAKMP_NEXT_NONE))
+    if (st->st_pfs_group != NULL && r!=NULL) {
+	if (!justship_KE(&st->st_gr
+			 , &md->rbody
+			 , id_pd != NULL? ISAKMP_NEXT_ID : ISAKMP_NEXT_NONE))
 	    return STF_INTERNAL_ERROR;
+
+	finish_dh_secret(st, r);
 	
-	stat = perform_dh_secret(st, RESPONDER, st->st_pfs_group->group);
-	if(stat != STF_OK) {
-	    return stat;
-	}
     }
     
     /* [ IDci, IDcr ] out */
@@ -2092,11 +2131,18 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
  * (see RFC 2409 "IKE" 5.5)
  * Installs inbound and outbound IPsec SAs, routing, etc.
  */
+static stf_status
+quick_inR1_outI2_cryptotail(struct dh_continuation *dh
+			    , struct pluto_crypto_req *r);
+static void
+quick_inR1_outI2_continue(struct pluto_crypto_req_cont *pcrc
+			  , struct pluto_crypto_req *r
+			  , err_t ugh);
+
 stf_status
 quick_inR1_outI2(struct msg_digest *md)
 {
     struct state *const st = md->st;
-    const struct connection *c = st->st_connection;
 
     /* HASH(2) in */
     CHECK_QUICK_HASH(md
@@ -2118,20 +2164,77 @@ quick_inR1_outI2(struct msg_digest *md)
     /* [ KE ] in (for PFS) */
     RETURN_STF_FAILURE(accept_PFS_KE(md, &st->st_gr, "Gr", "Quick Mode R1"));
 
-    if (st->st_pfs_group != NULL) {
-	stf_status stat;
+    if(st->st_pfs_group) {
+	struct dh_continuation *dh = alloc_thing(struct dh_continuation
+						 , "quick outI2 DH");
 
-	stat = perform_dh_secret(st, INITIATOR, st->st_pfs_group->group);
-	if(stat != STF_OK) {
-	    return stat;
-	}
+	/* set up DH calculation */
+	dh->md = md;
+	set_suspended(st, md);
+	dh->dh_pcrc.pcrc_func = quick_inR1_outI2_continue;
+	return start_dh_secret(&dh->dh_pcrc, st
+			       , st->st_import
+			       , INITIATOR
+			       , st->st_pfs_group->group);
+    } else {
+	/* just call the tail function */
+	struct dh_continuation dh;
+
+	dh.md=md;
+	return quick_inR1_outI2_cryptotail(&dh, NULL);
+    }
+}
+
+static void
+quick_inR1_outI2_continue(struct pluto_crypto_req_cont *pcrc
+			  , struct pluto_crypto_req *r
+			  , err_t ugh)
+{
+    struct dh_continuation *dh = (struct dh_continuation *)pcrc;
+    struct msg_digest *md = dh->md;
+    struct state *const st = md->st;
+    stf_status e;
+
+    DBG(DBG_CONTROLMORE
+	, DBG_log("quick inI1_outR1: calculated ke+nonce, calculating DH"));
+
+    /* XXX should check out ugh */
+    passert(ugh == NULL);
+    passert(cur_state == NULL);
+    passert(st != NULL);
+
+    passert(st->st_connection != NULL);
+
+    set_cur_state(st);	/* we must reset before exit */
+    st->st_calculating=FALSE;
+    set_suspended(st, NULL);
+
+    e = quick_inR1_outI2_cryptotail(dh, r);
+
+    if(dh->md != NULL) {
+	complete_state_transition(&dh->md, e);
+	if(dh->md) release_md(dh->md);
+    }
+    reset_cur_state();
+}
+
+stf_status
+quick_inR1_outI2_cryptotail(struct dh_continuation *dh
+			    , struct pluto_crypto_req *r)
+{
+    struct msg_digest *md = dh->md;
+    struct state *st = md->st;
+    struct connection *c = st->st_connection;
+
+    if (st->st_pfs_group != NULL && r!=NULL) {
+	finish_dh_secret(st, r);
     }
 
 #ifdef NAT_TRAVERSAL
-	if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
-	    (st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
-	    nat_traversal_natoa_lookup(md, &st->hidden_variables);
-	}
+    if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
+	(st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
+	nat_traversal_natoa_lookup(md, &st->hidden_variables);
+    }
 #endif
 
     /* [ IDci, IDcr ] in; these must match what we sent */
@@ -2212,39 +2315,13 @@ quick_inR1_outI2(struct msg_digest *md)
 
     /* HDR* out done */
 
-    /* HASH(3) out -- sometimes, we add more content */
+    /* HASH(3) out -- since this is the only content, no passes needed */
     {
 	u_char	/* set by START_HASH_PAYLOAD: */
 	    *r_hashval,	/* where in reply to jam hash value */
 	    *r_hash_start;	/* start of what is to be hashed */
 
-#ifdef IMPAIR_UNALIGNED_I2_MSG
-	{
-	    char *padstr=getenv("PLUTO_UNALIGNED_I2_MSG");
-	    
-	    if(padstr) {
-		pb_stream vid_pbs;
-		int padsize;
-		padsize = strtoul(padstr, NULL, 0);
-		
-		openswan_log("inserting fake VID payload of %u size", padsize);
-		START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_VID);
-		
-		if (!out_generic(ISAKMP_NEXT_NONE,
-				 &isakmp_vendor_id_desc, &md->rbody, &vid_pbs))
-		    return STF_INTERNAL_ERROR;
-		
-		if (!out_zero(padsize, &vid_pbs, "Filler VID"))
-		    return STF_INTERNAL_ERROR;
-		
-		close_output_pbs(&vid_pbs);
-	    } else {
-		START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_NONE);
-	    }
-	}
-#else
 	START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_NONE);
-#endif
 
 #ifdef TPM
 	{
@@ -2255,7 +2332,6 @@ quick_inR1_outI2(struct msg_digest *md)
 	    r_hashval = tpm_relocateHash(pbs);	
 	}
 #endif
-
 	(void)quick_mode_hash3(r_hashval, st);
     }
 
