@@ -795,7 +795,7 @@ pfkey_raw_eroute(const ip_address *this_host
 		 , ipsec_spi_t spi
 		 , unsigned int proto UNUSED
 		 , unsigned int transport_proto
-		 , unsigned int satype
+		 , enum eroute_type esatype
 		 , const struct pfkey_proto_info *proto_info UNUSED
 		 , time_t use_lifetime UNUSED
 		 , enum pluto_sadb_operations op
@@ -811,6 +811,7 @@ pfkey_raw_eroute(const ip_address *this_host
 
     int sport = ntohs(portof(&this_client->addr));
     int dport = ntohs(portof(&that_client->addr));
+    int satype;
 
     networkof(this_client, &sflow_ska);
     maskof(this_client, &smask_ska);
@@ -820,7 +821,31 @@ pfkey_raw_eroute(const ip_address *this_host
     maskof(that_client, &dmask_ska);
     setportof(dport ? ~0:0, &dmask_ska);
 
+    switch(esatype) {
+    case ET_UNSPEC:
+	    satype = K_SADB_SATYPE_UNSPEC;
+	    break;
 
+    case ET_AH:
+	    satype = K_SADB_SATYPE_AH;
+	    break;
+
+    case ET_ESP:
+	    satype = K_SADB_SATYPE_ESP;
+	    break;
+
+    case ET_IPCOMP:
+	    satype = K_SADB_X_SATYPE_COMP;
+	    break;
+
+    case ET_INT:
+	    satype = K_SADB_X_SATYPE_INT;
+	    break;
+
+    case ET_IPIP:
+	    satype = K_SADB_X_SATYPE_IPIP;
+	    break;
+    }
 
     if (!pfkey_msg_start(klips_op & KLIPS_OP_MASK, satype
 		       , "pfkey_msg_hdr flow", text_said, extensions))
@@ -828,10 +853,6 @@ pfkey_raw_eroute(const ip_address *this_host
 	return FALSE;
     }
 
-#if 0
-    DBG_log("klips pfkey op = %u / %u (ERO_DELETE=%u)", op, klips_op, ERO_DELETE);
-#endif
-
     if (op != ERO_DELETE)
     {
 	if (!(pfkey_build(pfkey_sa_build(&extensions[K_SADB_EXT_SA]
@@ -890,7 +911,9 @@ bool pfkey_add_sa(const struct kernel_sa *sa, bool replace)
     pfkey_buf pfb;
     bool success = FALSE;
 
-    success = pfkey_msg_start(replace ? K_SADB_UPDATE : K_SADB_ADD, sa->satype
+    int klips_satype = proto2satype(sa->esatype);
+
+    success = pfkey_msg_start(replace ? K_SADB_UPDATE : K_SADB_ADD, klips_satype
 			      , "pfkey_msg_hdr Add SA"
 			      , sa->text_said, extensions);
 
@@ -1018,8 +1041,11 @@ bool pfkey_add_sa(const struct kernel_sa *sa, bool replace)
 bool pfkey_grp_sa(const struct kernel_sa *sa0, const struct kernel_sa *sa1)
 {
     struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
+    int klips_satype0 = proto2satype(sa0->esatype);
+    int klips_satype1 = proto2satype(sa1->esatype);
 
-    return pfkey_msg_start(K_SADB_X_GRPSA, sa1->satype
+
+    return pfkey_msg_start(K_SADB_X_GRPSA, klips_satype1
 	, "pfkey_msg_hdr group", sa1->text_said, extensions)
 
     && pfkey_build(pfkey_sa_build(&extensions[K_SADB_EXT_SA]
@@ -1032,7 +1058,7 @@ bool pfkey_grp_sa(const struct kernel_sa *sa0, const struct kernel_sa *sa1)
 	, "pfkey_addr_d group", sa1->text_said, extensions)
 
     && pfkey_build(pfkey_x_satype_build(&extensions[K_SADB_X_EXT_SATYPE2]
-	    , sa0->satype)
+	    , klips_satype0)
 	, "pfkey_satype group", sa0->text_said, extensions)
 
     && pfkey_build(pfkey_sa_build(&extensions[K_SADB_X_EXT_SA2]
@@ -1170,7 +1196,7 @@ pfkey_shunt_eroute(struct connection *c
             esr->routing = RT_ROUTED_PROSPECTIVE;
             return pfkey_shunt_eroute(ue, esr
 				      , RT_ROUTED_PROSPECTIVE
-				      , (K_SADB_X_ADDFLOW | (SADB_X_SAFLAGS_REPLACEFLOW << KLIPS_OP_FLAG_SHIFT))
+				      , ERO_REPLACE
 				      , "restoring eclipsed");
         }
     }
@@ -1198,17 +1224,12 @@ pfkey_shunt_eroute(struct connection *c
     {
       const ip_address *peer = &sr->that.host_addr;
       char buf2[256];
-      const struct af_info *fam = aftoinfo(addrtypeof(peer));
-      
-      if(fam == NULL) {
-	      fam=aftoinfo(AF_INET);
-      }
       
       snprintf(buf2, sizeof(buf2)
 	       , "eroute_connection %s", opname);
 
       return pfkey_raw_eroute(&sr->this.host_addr, &sr->this.client
-			      , fam->any
+			      , aftoinfo(addrtypeof(peer))->any
 			      , &sr->that.client
 			      , htonl(spi)
 			      , SA_INT
@@ -1247,7 +1268,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_ah.attrs.spi;
         inner_proto = SA_AH;
-        inner_satype = SADB_SATYPE_AH;
+        inner_satype = ET_AH;
 
         i--;
         proto_info[i].proto = IPPROTO_AH;
@@ -1260,7 +1281,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_esp.attrs.spi;
         inner_proto = SA_ESP;
-        inner_satype = SADB_SATYPE_ESP;
+        inner_satype = ET_ESP;
 
         i--;
         proto_info[i].proto = IPPROTO_ESP;
@@ -1273,7 +1294,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_ipcomp.attrs.spi;
         inner_proto = SA_COMP;
-        inner_satype = SADB_X_SATYPE_COMP;
+        inner_satype = ET_IPCOMP;
 
         i--;
         proto_info[i].proto = IPPROTO_COMP;
@@ -1293,7 +1314,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
 
         inner_spi = st->st_tunnel_out_spi;
         inner_proto = SA_IPIP;
-        inner_satype = SADB_X_SATYPE_IPIP;
+        inner_satype = ET_IPIP;
 
         proto_info[i].encapsulation = ENCAPSULATION_MODE_TUNNEL;
         for (j = i + 1; proto_info[j].proto; j++)
@@ -1775,3 +1796,10 @@ pfkey_plumb_mast_device(int mast_dev)
 
 
 #endif /* KLIPS */
+
+/*
+ * Local Variables:
+ * c-basic-offset:4
+ * c-style: pluto
+ * End:
+ */
