@@ -35,7 +35,11 @@
 #include <sys/sysctl.h>
 #endif
 
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include <signal.h>
+#include <sys/wait.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
@@ -52,17 +56,15 @@
 #include "rnd.h"
 #include "pluto_crypt.h"
 
-#ifdef HAVE_OCF
+#ifdef HAVE_OCF_AND_OPENSSL
 #include "id.h"
 #include "pgp.h"
 #include "x509.h"
 #include "certs.h"
 #include "keys.h"
-#include "ocf_pk.h"
+#include "ocf_cryptodev.h"
 #endif
 
-TAILQ_HEAD(req_queue, pluto_crypto_req_cont);
-
 struct pluto_crypto_worker {
     int   pcw_helpernum;
     pid_t pcw_pid;
@@ -72,11 +74,11 @@ struct pluto_crypto_worker {
     int   pcw_maxcritwork;  /* how many critical things can be queued */
     bool  pcw_dead;         /* worker is dead, waiting for reap */
     bool  pcw_reaped;       /* worker has been reaped, waiting for dealloc */
-    struct req_queue pcw_active;
+    struct pluto_crypto_req_cont *pcw_cont;
 };
 
-static struct req_queue backlog;
-static int       backlogqueue_len=0;
+static struct pluto_crypto_req_cont *backlogqueue;
+static int                           backlogqueue_len;
 
 static void init_crypto_helper(struct pluto_crypto_worker *w, int n);
 static void cleanup_crypto_helper(struct pluto_crypto_worker *w, int status);
@@ -89,6 +91,8 @@ int pc_workers_cnt = 0;
 int pc_worker_num;
 pcr_req_id pcw_id;
 
+bool using_vulcan_hack=FALSE;
+
 /* local in child */
 int pc_helper_num=-1;
 
@@ -175,8 +179,7 @@ helper_passert_fail(const char *pred_str
 
 void pluto_crypto_helper(int fd, int helpernum)
 {
-    FILE *in  = fdopen(fd, "rb");
-    FILE *out = fdopen(fd, "wb");
+    FILE *io = fdopen(fd, "ab+");
     long reqbuf[PCR_REQ_SIZE/sizeof(long)];
     struct pluto_crypto_req *r;
 
@@ -188,10 +191,10 @@ void pluto_crypto_helper(int fd, int helpernum)
     setpriority(PRIO_PROCESS, 0, 10);
 
     DBG(DBG_CONTROL, DBG_log("helper %d waiting on fd: %d"
-			     , helpernum, fileno(in)));
+			     , helpernum, fileno(io)));
 
     memset(reqbuf, 0, sizeof(reqbuf));
-    while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, in) == 1) {
+    while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, io) == 1) {
 	int restlen;
 	int actnum;
 	unsigned char *reqrest = ((unsigned char *)reqbuf)+sizeof(r->pcr_len);
@@ -200,15 +203,10 @@ void pluto_crypto_helper(int fd, int helpernum)
 	restlen = r->pcr_len-sizeof(r->pcr_len);
 	
 	passert(restlen < (signed)PCR_REQ_SIZE);
-	passert(restlen > 0);
 
-	actnum = fread(reqrest, 1, restlen, in);
+	actnum = fread(reqrest, restlen, 1, io);
 	/* okay, got a basic size, read the rest of it */
-
-	DBG(DBG_CONTROL, DBG_log("helper %d read %d+4/%d bytesfd: %d"
-				 , helpernum, actnum, r->pcr_len, fileno(in)));
-
-	if(actnum != restlen) {
+	if(actnum != 1) {
 	    /* faulty read. die, parent will restart us */
 	    loglog(RC_LOG_SERIOUS, "cryptographic helper(%d) fread(%d)=%d failed: %s\n",
 		   getpid(), restlen, actnum, strerror(errno));
@@ -225,8 +223,7 @@ void pluto_crypto_helper(int fd, int helpernum)
 
 	pluto_do_crypto_op(r);
 
-	actnum = fwrite((unsigned char *)r, r->pcr_len, 1, out);
-	fflush(out);
+	actnum = fwrite((unsigned char *)r, r->pcr_len, 1, io);
 
 	if(actnum != 1) {
 	    loglog(RC_LOG_SERIOUS, "failed to write answer: %d", actnum);
@@ -235,53 +232,16 @@ void pluto_crypto_helper(int fd, int helpernum)
 	memset(reqbuf, 0, sizeof(reqbuf));
     }
 
-    if(!feof(in)) {
-	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(in)));
+    if(!feof(io)) {
+	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(io)));
     }
 
     /* probably normal EOF */
-    fclose(in);
-    fclose(out);
+    fclose(io);
     exit(0);
 }
 
 
-/* send the request, make sure it all goes down. */
-static bool crypto_write_request(struct pluto_crypto_worker *w
-				 ,struct pluto_crypto_req *r)
-{
-    unsigned char *wdat = (unsigned char *)r;
-    int wlen = r->pcr_len;
-    int cnt;
-    
-    DBG(DBG_CONTROL
-	, DBG_log("asking helper %d to do %s op on seq: %u (len=%u, pcw_work=%d)"
-		  , w->pcw_helpernum
-		  , enum_show(&pluto_cryptoop_names, r->pcr_type)
-		  , r->pcr_id, r->pcr_len, w->pcw_work+1));
-
-    do {
-	errno=0;
-	cnt = write(w->pcw_pipe, wdat, wlen);
-	
-	if(cnt <= 0) {
-	    openswan_log("write to helper failed: cnt=%d err=%s\n",
-			 cnt, strerror(errno));
-	    return FALSE;
-	}
-	if(DBGP(DBG_CONTROL) || cnt != wlen) {
-	    DBG_log("crypto helper write of request: cnt=%d<wlen=%d. \n", cnt, wlen);
-	}
-
-	wlen -= cnt;
-	wdat += cnt;
-
-    } while(wlen > 0);
-
-    return TRUE;
-}
-
-
 /*
  * this function is called with a request to do some cryptographic operations
  * along with a continuation structure, which will be used to deal with
@@ -319,66 +279,64 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     cn->pcrc_id = r->pcr_id;
     cn->pcrc_pcr = r;
 
-    /* find an available worker */
-    cnt = pc_workers_cnt;
-    do {
-	pc_worker_num++;
- 	if(pc_worker_num >= pc_workers_cnt) {
- 	    pc_worker_num = 0;
- 	}
-	w = &pc_workers[pc_worker_num];
+    pc_worker_num++;
+    if(pc_worker_num >= pc_workers_cnt) {
+	pc_worker_num = 0;
+    }
 
- 	DBG(DBG_CONTROL
- 	    , DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
- 		      pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
+    cnt = pc_workers_cnt;
+
+    /* find an available worker, restarting one if it was found to be dead */
+    w = &pc_workers[pc_worker_num];
+
+    DBG(DBG_CONTROL
+	, DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
+		  pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
+    
+    while((w->pcw_dead || (w->pcw_work >= w->pcw_maxbasicwork))
+	  && --cnt > 0) {
+	
+	pc_worker_num++;
+	w = &pc_workers[pc_worker_num];
 
 	/* see if there is something to clean up after */
 	if(w->pcw_dead      == TRUE
 	   && w->pcw_reaped == TRUE) {
 	    cleanup_crypto_helper(w, 0);
- 	    DBG(DBG_CONTROL
- 		, DBG_log("clnup %d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
- 			  pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
 	}
-    } while(((w->pcw_work >= w->pcw_maxbasicwork))
- 	    && --cnt > 0);
+	DBG(DBG_CONTROL
+	    , DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
+		      pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
+    }
 
     if(cnt == 0 && r->pcr_pcim > pcim_ongoing_crypto) {
 	cnt = pc_workers_cnt;
- 	while((w->pcw_work >= w->pcw_maxcritwork)
+	while((w->pcw_dead || (w->pcw_work >= w->pcw_maxcritwork))
 	      && --cnt > 0) {
 	
- 	    /* find an available worker */
 	    pc_worker_num++;
- 	    if(pc_worker_num >= pc_workers_cnt) {
- 		pc_worker_num = 0;
-  	    }
-
 	    w = &pc_workers[pc_worker_num];
+
 	    /* see if there is something to clean up after */
 	    if(w->pcw_dead      == TRUE
 	       && w->pcw_reaped == TRUE) {
 		cleanup_crypto_helper(w, 0);
 	    }
+	    DBG(DBG_CONTROL
+		, DBG_log("crit %d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
+			  pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
 	}
-	DBG(DBG_CONTROL
-	    , DBG_log("crit %d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
-		      pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
     }
 
     if(cnt == 0 && r->pcr_pcim >= pcim_demand_crypto) {
 	/* it is very important. Put it all on a queue for later */
-	
-	TAILQ_INSERT_TAIL(&backlog, cn, pcrc_list);
-
-	/* copy the request */
-	r = clone_bytes(r, r->pcr_len, "saved cryptorequest");
-	cn->pcrc_pcr = r;
+	cn->pcrc_next = backlogqueue;
+	backlogqueue  = cn;
 	
 	backlogqueue_len++;
 	DBG(DBG_CONTROL
-	    , DBG_log("critical demand crypto operation queued on backlog as %d'th item, id: q#%u"
-		      , backlogqueue_len, r->pcr_id));
+	    , DBG_log("critical demand crypto operation queued as item %d"
+		      , backlogqueue_len));
 	*toomuch = FALSE;
 	return NULL;
     }
@@ -386,7 +344,8 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     if(cnt == 0) {
 	/* didn't find any workers */
 	DBG(DBG_CONTROL
-	    , DBG_log("failed to find any available worker"));
+	    , DBG_log("failed to find any available worker (import=%s)"
+		      , enum_name(&pluto_cryptoimportance_names,r->pcr_pcim)));
 
 	*toomuch = TRUE;
 	return "failed to find any available worker";
@@ -404,18 +363,25 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     }
 
     /* link it to the active worker list */
-    TAILQ_INSERT_TAIL(&w->pcw_active, cn, pcrc_list);
+    cn->pcrc_next = w->pcw_cont;
+    w->pcw_cont = cn;
 
     passert(w->pcw_pid != -1);
     passert(w->pcw_pipe != -1);
     passert(w->pcw_work < w->pcw_maxcritwork);
     
-    if(!crypto_write_request(w, r)) {
-	openswan_log("failed to write crypto request: %s\n",
-		     strerror(errno));
+    DBG(DBG_CONTROL
+	, DBG_log("asking helper %d to do %s op on seq: %u"
+		  , w->pcw_helpernum
+		  , enum_show(&pluto_cryptoop_names, r->pcr_type)
+		  , r->pcr_id));
+
+    /* send the request, and then mark the work as having more work */
+    cnt = write(w->pcw_pipe, r, r->pcr_len);
+    if(cnt == -1) {
 	return "failed to write";
-    }
-	
+    } 
+
     w->pcw_work++;
     *toomuch = FALSE;
     return NULL;
@@ -430,20 +396,21 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
     struct pluto_crypto_req_cont *cn;
 
     if(backlogqueue_len > 0) {
+	int cnt;
 
-	passert(backlog.tqh_first != NULL);
-	cn = backlog.tqh_first;
-	TAILQ_REMOVE(&backlog, cn, pcrc_list);
+	passert(backlogqueue != NULL);
 	
+	cn = backlogqueue;
+	backlogqueue = cn->pcrc_next;
 	backlogqueue_len--;
 	
 	r = cn->pcrc_pcr;
       
 	DBG(DBG_CONTROL
-	    , DBG_log("removing backlog item id: q#%u from queue: %d left"
+	    , DBG_log("removing backlog item (%d) from queue: %d left"
 		      , r->pcr_id, backlogqueue_len));
 
-	/* w points to a worker. Make sure it is live */
+	/* w points to a work. Make sure it is live */
 	if(w->pcw_pid == -1) {
 	    init_crypto_helper(w, pc_worker_num);
 	    if(w->pcw_pid == -1) {
@@ -456,21 +423,26 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
 	}
 	
 	/* link it to the active worker list */
-	TAILQ_INSERT_TAIL(&w->pcw_active, cn, pcrc_list);
+	cn->pcrc_next = w->pcw_cont;
+	w->pcw_cont = cn;
 	
 	passert(w->pcw_pid != -1);
 	passert(w->pcw_pipe != -1);
 	passert(w->pcw_work > 0);
     
-	/* send the request, and then mark the worker as having more work */
-	if(!crypto_write_request(w, r)) {
+	DBG(DBG_CONTROL
+	    , DBG_log("asking helper %d to do %s op on seq: %u"
+		      , w->pcw_helpernum
+		      , enum_show(&pluto_cryptoop_names, r->pcr_type)
+		      , r->pcr_id));
+	
+	/* send the request, and then mark the work as having more work */
+	cnt = write(w->pcw_pipe, r, r->pcr_len);
+	if(cnt == -1) {
 	    /* XXX invoke callback with failure */
 	    passert(0);
 	    return;
 	} 
-
-	/* if it was on the backlog, it was saved, free it */
-	pfree(r);
 	
 	w->pcw_work++;
     }
@@ -503,12 +475,15 @@ void delete_cryptographic_continuation(struct state *st)
 
     for(i=0; i<pc_workers_cnt; i++) {
 	struct pluto_crypto_worker *w = &pc_workers[i];
-	struct pluto_crypto_req_cont *cn;
+	struct pluto_crypto_req_cont *cn, **cnp;
 
-	for(cn = w->pcw_active.tqh_first;
-	    cn!=NULL && st->st_serialno != cn->pcrc_serialno;
-	    cn = cn->pcrc_list.tqe_next);
-		
+	cn = w->pcw_cont;
+	cnp = &w->pcw_cont;
+	while(cn && st->st_serialno != cn->pcrc_serialno) {
+	    cnp = &cn->pcrc_next;
+	    cn = cn->pcrc_next;
+	}
+	
 	if(cn == NULL) {
 	    DBG(DBG_CRYPT, DBG_log("no suspended cryptographic state for %lu\n"
 				   , st->st_serialno));
@@ -516,7 +491,8 @@ void delete_cryptographic_continuation(struct state *st)
 	}
 
 	/* unlink it, and free it */
-	TAILQ_REMOVE(&w->pcw_active, cn, pcrc_list);
+	*cnp = cn->pcrc_next;
+	cn->pcrc_next = NULL;
  
 	if(cn->pcrc_free) {
 	    /*
@@ -547,12 +523,14 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     struct pluto_crypto_req *r;
     int restlen;
     int actlen;
-    struct pluto_crypto_req_cont *cn;
+    struct pluto_crypto_req_cont *cn, **cnp;
 
-    DBG(DBG_CRYPT|DBG_CONTROL
-	, DBG_log("helper %u has finished work (cnt now %d)"
-		  ,w->pcw_helpernum
-		  ,w->pcw_work));
+    /* we can accept more work now that we are about to read from the pipe */
+    w->pcw_work--;
+
+    DBG(DBG_CRYPT, DBG_log("helper %u has work (cnt now %d)"
+			   ,w->pcw_helpernum
+			   ,w->pcw_work));
 
     /* read from the pipe */
     actlen = read(w->pcw_pipe, (char *)reqbuf, sizeof(r->pcr_len));
@@ -572,9 +550,6 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
 	return;
     }
 
-    /* we can accept more work now that we have read from the pipe */
-    w->pcw_work--;
-
     r = (struct pluto_crypto_req *)reqbuf;
 
     if(r->pcr_len > sizeof(reqbuf)) {
@@ -610,9 +585,9 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
 	inloc   += actlen;
     }
 
-    DBG(DBG_CRYPT|DBG_CONTROL, DBG_log("helper %u replies to id: q#%u"
-				       ,w->pcw_helpernum
-				       ,r->pcr_id));
+    DBG(DBG_CRYPT, DBG_log("helper %u replies to sequence %u"
+			   ,w->pcw_helpernum
+			   ,r->pcr_id));
 
     /*
      * if there is work queued, then send it off after reading, since this
@@ -621,10 +596,13 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     crypto_send_backlog(w);
 
     /* now match up request to continuation, and invoke it */
-    for(cn = w->pcw_active.tqh_first;
-	cn!=NULL && r->pcr_id != cn->pcrc_id;
-	cn = cn->pcrc_list.tqe_next);
-		
+    cn = w->pcw_cont;
+    cnp = &w->pcw_cont;
+    while(cn && r->pcr_id != cn->pcrc_id) {
+	cnp = &cn->pcrc_next;
+	cn = cn->pcrc_next;
+    }
+
     if(cn == NULL) {
 	loglog(RC_LOG_SERIOUS
 	       , "failed to find continuation associated with req %u\n",
@@ -633,7 +611,8 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     }
 
     /* unlink it */
-    TAILQ_REMOVE(&w->pcw_active, cn, pcrc_list);
+    *cnp = cn->pcrc_next;
+    cn->pcrc_next = NULL;
  
     passert(cn->pcrc_func != NULL);
 
@@ -672,7 +651,6 @@ static void init_crypto_helper(struct pluto_crypto_worker *w, int n)
     w->pcw_work     = 0;
     w->pcw_reaped = FALSE;
     w->pcw_dead   = FALSE;
-    TAILQ_INIT(&w->pcw_active);
 
     /* set the send/received queue length to be at least maxcritwork
      * times sizeof(pluto_crypto_req) in size
@@ -706,18 +684,24 @@ static void init_crypto_helper(struct pluto_crypto_worker *w, int n)
 	int fd;
 	int maxfd;
 	struct rlimit nf;
-	int i;
 
+#ifdef HAVE_SETPROCTITLE
+	setproctitle("pluto helper %s #%3d   ", pluto_ifn_inst, n);
+#else
 	/* diddle with our proc title */
-	memset(global_argv[0], '\0', strlen(global_argv[0])+1);
-	sprintf(global_argv[0], "pluto helper %s #%3d   ", pluto_ifn_inst, n);
-	for(i = 1; i < global_argc; i++) {
-	    if(global_argv[i]) {
-		int l = strlen(global_argv[i]);
-		memset(global_argv[i], '\0', l);
+	{
+	    int i;
+	    memset(global_argv[0], '\0', strlen(global_argv[0])+1);
+	    sprintf(global_argv[0], "pluto helper %s #%3d   ", pluto_ifn_inst, n);
+	    for(i = 1; i < global_argc; i++) {
+		if(global_argv[i]) {
+		    int l = strlen(global_argv[i]);
+		    memset(global_argv[i], '\0', l);
+		}
+		global_argv[i]=NULL;
 	    }
-	    global_argv[i]=NULL;
 	}
+#endif
 
 	if(getenv("PLUTO_CRYPTO_HELPER_DEBUG")) {
 	    sprintf(global_argv[0], "pluto helper %s #%3d (waiting for GDB) "
@@ -740,14 +724,30 @@ static void init_crypto_helper(struct pluto_crypto_worker *w, int n)
 	
 	pluto_init_log();
 	init_rnd_pool();
-#ifdef HAVE_OCF
+#ifdef HAVE_OCF_AND_OPENSSL
 	load_cryptodev();
 #endif
 	free_preshared_secrets();
 	openswan_passert_fail = helper_passert_fail;
 	debug_prefix='!';
 
+#ifdef VULCAN_PK
+	if(using_vulcan_hack) {
+	    vulcanpk_mapping = mapvulcanpk();
+	    /* initialize chip */
+	    vulcanpk_init(vulcanpk_mapping);
+	}
+#endif	
+
 	pluto_crypto_helper(fds[1], n);
+
+#ifdef VULCAN_PK
+	if(using_vulcan_hack) {
+	    /* shut down */
+	    unmapvulcanpk(vulcanpk_mapping);
+	}
+#endif
+
 	exit(0);
 	/* NOTREACHED */
     }
@@ -778,13 +778,16 @@ static void cleanup_crypto_helper(struct pluto_crypto_worker *w
 				  , int status)
 {
     if(w->pcw_pipe) {
-	loglog(RC_LOG_SERIOUS, "closing helper(%u) pid=%d fd=%d exit=%d"
-	       , w->pcw_helpernum, w->pcw_pid, w->pcw_pipe, status);
+	loglog(RC_LOG_SERIOUS, "closing helper(%u) pid=%d fd=%d killed=%s(%d%s) exit=%d"
+	       , w->pcw_helpernum, w->pcw_pid, w->pcw_pipe
+	       , WIFEXITED(status) ? "no" : "yes"
+	       , WIFEXITED(status) ? 0 : WTERMSIG(status) 
+	       , WIFEXITED(status) && WCOREDUMP(status) ? ":core" : ""
+	       , WEXITSTATUS(status));
 	close(w->pcw_pipe);
     }
 
     w->pcw_pid = -1;
-    w->pcw_work = 0; /* ?!? */
     w->pcw_reaped = FALSE;
     w->pcw_dead   = FALSE;   /* marking is not dead lets it live again */
 }
@@ -806,7 +809,24 @@ void init_crypto_helpers(int nhelpers)
     pc_workers_cnt = 0;
     pcw_id = 1;
 
-    TAILQ_INIT(&backlog);
+#ifdef VULCAN_PK
+    if(please_use_vulcan_hack) {
+	int vfd;
+
+	vfd = open("/dev/vulcanpk", O_RDWR);
+	if(vfd != -1) {
+	    calc_dh_shared = calc_dh_shared_vulcanpk;
+	    using_vulcan_hack = TRUE;
+	    openswan_log("Enabling direct Vulcan Public Key interface\n");
+	    close(vfd);
+
+	    if(nhelpers == -1) {
+		/* only need one helper for this situation, in theory */
+		nhelpers = 1;
+	    }
+	}
+    }
+#endif
 
     /* find out how many CPUs there are, if nhelpers is -1 */
     /* if nhelpers == 0, then we do all the work ourselves */
@@ -888,120 +908,6 @@ int pluto_crypto_helper_ready(fd_set *readfds)
     return ndes;
 }
 
-
-/*
- * invoke helper to do DH work.
- */
-stf_status perform_dh_secretiv(struct state *st
-			     , enum phase1_role init       /* TRUE=g_init,FALSE=g_r */
-			     , u_int16_t oakley_group)
-{
-    struct pluto_crypto_req r;
-    struct pcr_skeyid_q *dhq = &r.pcr_d.dhq;
-    struct pcr_skeyid_r *dhr = &r.pcr_d.dhr;
-    const chunk_t *pss = get_preshared_secret(st->st_connection);
-
-    passert(st->st_sec_in_use);
-
-    dhq->thespace.start = 0;
-    dhq->thespace.len   = sizeof(dhq->space);
-
-    /* convert appropriate data to dhq */
-    dhq->auth = st->st_oakley.auth;
-    dhq->hash = st->st_oakley.hash;
-    dhq->oakley_group = oakley_group;
-    dhq->init = init;
-    dhq->keysize = st->st_oakley.enckeylen/BITS_PER_BYTE; 
-
-    if(pss) {
-	pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->pss, *pss);
-    }
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->ni,  st->st_ni);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->nr,  st->st_nr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gi,  st->st_gi);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gr,  st->st_gr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space
-			   , &dhq->secret, st->st_sec_chunk);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->icookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->icookie)
-	   , st->st_icookie, COOKIE_SIZE);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->rcookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->rcookie)
-	   , st->st_rcookie, COOKIE_SIZE);
-
-    calc_dh_iv(&r);
-
-    clonetochunk(st->st_shared,   wire_chunk_ptr(dhr, &(dhr->shared))
-		 , dhr->shared.len,   "calculated shared secret");
-    clonetochunk(st->st_skeyid,   wire_chunk_ptr(dhr, &(dhr->skeyid))
-		 , dhr->skeyid.len,   "calculated skeyid secret");
-    clonetochunk(st->st_skeyid_d, wire_chunk_ptr(dhr, &(dhr->skeyid_d))
-		 , dhr->skeyid_d.len, "calculated skeyid_d secret");
-    clonetochunk(st->st_skeyid_a, wire_chunk_ptr(dhr, &(dhr->skeyid_a))
-		 , dhr->skeyid_a.len, "calculated skeyid_a secret");
-    clonetochunk(st->st_skeyid_e, wire_chunk_ptr(dhr, &(dhr->skeyid_e))
-		 , dhr->skeyid_e.len, "calculated skeyid_a secret");
-    clonetochunk(st->st_enc_key, wire_chunk_ptr(dhr, &(dhr->enc_key))
-		 , dhr->enc_key.len, "calculated key for phase 1");
-    
-    passert(dhr->new_iv.len <= MAX_DIGEST_LEN);
-    passert(dhr->new_iv.len > 0);
-    memcpy(st->st_new_iv, wire_chunk_ptr(dhr, &(dhr->new_iv)),dhr->new_iv.len);
-    st->st_new_iv_len = dhr->new_iv.len;
-
-    st->hidden_variables.st_skeyid_calculated = TRUE;
-    return STF_OK;
-}
-
-stf_status perform_dh_secret(struct state *st
-			     , enum phase1_role init      
-			     , u_int16_t oakley_group)
-{
-    struct pluto_crypto_req r;
-    struct pcr_skeyid_q *dhq = &r.pcr_d.dhq;
-    struct pcr_skeyid_r *dhr = &r.pcr_d.dhr;
-    const chunk_t *pss = get_preshared_secret(st->st_connection);
-
-    passert(st->st_sec_in_use);
-
-    dhq->thespace.start = 0;
-    dhq->thespace.len   = sizeof(dhq->space);
-
-    /* convert appropriate data to dhq */
-    dhq->auth = st->st_oakley.auth;
-    dhq->hash = st->st_oakley.hash;
-    dhq->oakley_group = oakley_group;
-    dhq->init = init;
-    dhq->keysize = st->st_oakley.enckeylen/BITS_PER_BYTE; 
-
-    if(pss) {
-	pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->pss, *pss);
-    }
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->ni,  st->st_ni);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->nr,  st->st_nr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gi,  st->st_gi);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gr,  st->st_gr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space
-			   , &dhq->secret, st->st_sec_chunk);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->icookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->icookie)
-	   , st->st_icookie, COOKIE_SIZE);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->rcookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->rcookie)
-	   , st->st_rcookie, COOKIE_SIZE);
-
-    calc_dh(&r);
-
-    clonetochunk(st->st_shared,   wire_chunk_ptr(dhr, &(dhr->shared))
-		 , dhr->shared.len,   "calculated shared secret");
-    
-    return STF_OK;
-}
-
 /*
  * Local Variables:
  * c-basic-offset:4
