@@ -30,8 +30,6 @@
 #ifndef _PLUTO_CRYPT_H
 #define _PLUTO_CRYPT_H
 
-#include "crypto.h"
-
 typedef unsigned int pcr_req_id;
 
 typedef struct wire_chunk {
@@ -105,6 +103,7 @@ struct pcr_skeyid_r {
 
 struct pluto_crypto_req {
   size_t                     pcr_len;
+
   enum pluto_crypto_requests pcr_type;
   pcr_req_id                 pcr_id;
   enum crypto_importance     pcr_pcim;
@@ -123,7 +122,7 @@ typedef void (*crypto_req_func)(struct pluto_crypto_req_cont *
 				, err_t ugh);
 
 struct pluto_crypto_req_cont {
-	TAILQ_ENTRY(pluto_crypto_req_cont) pcrc_list;
+  struct pluto_crypto_req_cont *pcrc_next;
   struct pluto_crypto_req      *pcrc_pcr;
   so_serial_t                   pcrc_serialno;
   pcr_req_id                    pcrc_id;
@@ -170,16 +169,41 @@ extern void compute_dh_shared(struct state *st, const chunk_t g
 extern stf_status perform_dh(struct pluto_crypto_req_cont *cn, struct state *st);
 extern bool generate_skeyids_iv(struct state *st);
 
-extern stf_status perform_dh_secretiv(struct state *st
-				      , enum phase1_role  init  
-				      , u_int16_t oakley_group);
+extern stf_status start_dh_secretiv(struct pluto_crypto_req_cont *cn
+				    , struct state *st
+				    , enum crypto_importance importance
+				    , enum phase1_role init /* TRUE=g_init,FALSE=g_r */
+				    , u_int16_t oakley_group);
+
+extern void finish_dh_secretiv(struct state *st,
+			       struct pluto_crypto_req *r);
+
+extern stf_status start_dh_secret(struct pluto_crypto_req_cont *cn
+				  , struct state *st
+				  , enum crypto_importance importance
+				  , enum phase1_role init      
+				  , u_int16_t oakley_group);
+
+extern void finish_dh_secret(struct state *st,
+			     struct pluto_crypto_req *r);
 
-extern stf_status perform_dh_secret(struct state *st
-				    , enum phase1_role init
-				    , u_int16_t group);
 
 extern void calc_dh_iv(struct pluto_crypto_req *r);
 extern void calc_dh(struct pluto_crypto_req *r);
 
+typedef void (*calc_dh_shared_t)(chunk_t *shared, const chunk_t g
+				 , const chunk_t *secchunk
+				 , const struct oakley_group_desc *group);
+
+extern calc_dh_shared_t calc_dh_shared;
+
+#ifdef VULCAN_PK
+extern void calc_dh_shared_vulcanpk(chunk_t *shared, const chunk_t g
+				    , const chunk_t *secchunk
+				    , const struct oakley_group_desc *group);
+extern unsigned char *vulcanpk_mapping;
+#include "dev/hifn/vulcanpk_funcs.h"
+#endif
+
 #endif /* _PLUTO_CRYPT_H */
 
