@@ -86,8 +86,8 @@
 #include "tpm/tpm.h"
 #endif
 
-#ifdef HAVE_OCF
-#include "ocf_pk.h"
+#ifdef HAVE_OCF_AND_OPENSSL
+#include "ocf_cryptodev.h"
 #endif
 
 #ifndef IPSECDIR
@@ -115,6 +115,7 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--nofork]"
 	    " [--stderrlog]"
+	    " [--use-nostack]"         /* old --no_klips */
 	    " [--nocrsend]"
 	    " [--strictcrlpolicy]"
 	    " [--crlcheckinterval]"
@@ -123,7 +124,8 @@ usage(const char *mess)
 	    " [--use-auto]"
 	    " [--use-klips]"
 	    " [--use-netkey]"
-	    " [--use-nostack]"         /* old --no_klips */
+	    " [--use-mastklips]"
+	    " [--use-bsdkame]"
 	    " \\\n\t"
 	    "[--interface <ifname|ifaddr>]"
 	    " [--ikeport <port-number>]"
@@ -132,8 +134,6 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--perpeerlogbase <path>] [--perpeerlog]"
 	    " \\\n\t"
-	    "[--coredir <dirname>] [--noretransmits]"
-	    " \\\n\t"
 	    "[--secretsfile <secrets-file>]"
 	    " [--ipsecdir <ipsec-dir>]"
 	    " \\\n\t"
@@ -148,14 +148,12 @@ usage(const char *mess)
 	    " [--debug-crypt]"
 	    " [--debug-parsing]"
 	    " [--debug-emitting]"
+	    " [--debug-x509]"
 	    " \\\n\t"
 	    "[--debug-control]"
-	    "[--debug-lifecycle]"
 	    " [--debug-klips]"
 	    " [--debug-x509]"
 	    " [--debug-dns]"
-	    " [--debug-oppo]"
-	    " [--debug-oppoinfo]"
 	    " [--debug-dpd]"
 	    " [ --debug-private]"
 	    " [ --debug-pfkey]"
@@ -171,6 +169,10 @@ usage(const char *mess)
 	   " \\\n\t"
 	   "[--virtual_private <network_list>]"
 #endif
+#ifdef VULCAN_PK
+	   " \\\n\t"
+	    "[--usevulcanhack]"
+#endif	    
 	    "\n"
 	"Openswan %s\n"
 	, ipsec_version_code());
@@ -330,6 +332,8 @@ main(int argc, char **argv)
 	    { "usenetkey", no_argument, NULL, 'K' },
 	    { "use-netkey", no_argument, NULL, 'K' },
 	    { "use-mast",   no_argument, NULL, 'M' },
+	    { "use-mastklips", no_argument, NULL, 'M' },
+	    { "use-bsdkame",   no_argument, NULL, 'F' },
 	    { "interface", required_argument, NULL, 'i' },
 	    { "ikeport", required_argument, NULL, 'p' },
 	    { "ctlbase", required_argument, NULL, 'b' },
@@ -358,6 +362,9 @@ main(int argc, char **argv)
 #ifdef VIRTUAL_IP
 	    { "virtual_private", required_argument, NULL, '6' },
 #endif
+#ifdef VULCAN_PK
+	    { "usevulcanhack", no_argument, NULL, '7' },  /* 7956 */
+#endif	    
 	    { "nhelpers", required_argument, NULL, 'j' },
 #ifdef DEBUG
 	    { "debug-none", no_argument, NULL, 'N' },
@@ -372,7 +379,6 @@ main(int argc, char **argv)
 	    { "debug-klips", no_argument, NULL, DBG_KLIPS + DBG_OFFSET },
 	    { "debug-dns", no_argument, NULL, DBG_DNS + DBG_OFFSET },
 	    { "debug-oppo", no_argument, NULL, DBG_OPPO + DBG_OFFSET },
-	    { "debug-oppoinfo", no_argument, NULL, DBG_OPPOINFO + DBG_OFFSET },
 	    { "debug-controlmore", no_argument, NULL, DBG_CONTROLMORE + DBG_OFFSET },
 	    { "debug-dpd", no_argument, NULL, DBG_DPD + DBG_OFFSET },
             { "debug-x509", no_argument, NULL, DBG_X509 + DBG_OFFSET },
@@ -467,6 +473,10 @@ main(int argc, char **argv)
 	    kern_interface = USE_MASTKLIPS;
 	    continue;
 
+	case 'F':       /* --use-bsdkame */
+	    kern_interface = USE_BSDKAME;
+	    continue;
+
 	case 'K':       /* --use-netkey */
 	    kern_interface = USE_NETKEY;
 	    continue;
@@ -593,6 +603,11 @@ main(int argc, char **argv)
 	    base_debugging |= DBG_NATT;
 	    continue;
 #endif
+#ifdef VULCAN_PK
+        case '7':
+	    please_use_vulcan_hack=TRUE;
+	    continue;
+#endif
 #ifdef VIRTUAL_IP
 	case '6':	/* --virtual_private */
 	    virtual_private = optarg;
@@ -612,8 +627,11 @@ main(int argc, char **argv)
 	}
 	break;
     }
-    if (optind != argc)
+
+    if (optind != argc) {
+	fprintf(stderr, "Extra arguments: %s\n", argv[optind]);
 	usage("unexpected argument");
+    }
     reset_debugging();
 
     /* if a core dir was set, chdir there */
@@ -621,7 +639,13 @@ main(int argc, char **argv)
 	chdir(coredir);
     }
 
+#ifdef HAVE_SETPROCTITLE
+    setproctitle("pluto master process ");
+#endif
+
     oco = osw_init_options();
+    DBG(DBG_CONTROL, DBG_log("confddir set to %s\n", oco->confddir));
+
     lockfd = create_lock();
 
     /* select between logging methods */
@@ -745,14 +769,14 @@ main(int argc, char **argv)
         const char *v = init_pluto_vendorid();
 	const char *vc = ipsec_version_code();
 
-        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s) pid:%u"
-		     , vc
-		     , compile_time_interop_options
-		     , v, getpid());
+        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s)"
+            , vc
+            , compile_time_interop_options
+            , v);
 #else
-        openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
-		     , ipsec_version_code()
-		     , compile_time_interop_options, getpid());
+        openswan_log("Starting Pluto (Openswan Version %s%s)"
+            , ipsec_version_code()
+            , compile_time_interop_options);
 #endif
 
 	if((vc[0]=='c' && vc[1]=='v' && vc[2]=='s') ||
@@ -791,7 +815,7 @@ main(int argc, char **argv)
     init_connections();
     init_crypto();
     init_crypto_helpers(nhelpers);
-#ifdef HAVE_OCF
+#ifdef HAVE_OCF_AND_OPENSSL
     load_cryptodev();
 #endif
     init_demux();
