@@ -68,23 +68,15 @@
 #include "log.h"
 #include "timer.h"
 
-/*
- * we have removed /dev/hw_random, as it can produce very low quality
- * entropy. One must run rngd to verify the entropy and feed it into
- * /dev/random properly.
- *
- * You have been warned.
- *
- */
 static int random_fd = -1;
 const char *random_devices[]={
 #if defined(linux) 
-  "/dev/urandom",
-  "/dev/random"
+  "/dev/hw_random",
+  "/dev/urandom"
 #elif defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
   "/dev/urandom",
   "/dev/random"
-#elif defined(__OpenBSD__)
+#elif defined(__OpenBSD__) || defined(__FreeBSD__)
   "/dev/random"
 #elif defined(__CYGWIN__)
   "/dev/random"
