@@ -434,6 +434,14 @@ create_socket(struct raw_iface *ifp, const char *v_name, int port)
     udpfromto_init(fd);
 #endif
 
+    /* poke a hole for IKE messages in the IPsec layer */
+    if(kernel_ops->exceptsocket) {
+	if(!(*kernel_ops->exceptsocket)(fd, AF_INET)) {
+	    close(fd);
+	    return -1;
+	}
+    }
+
     return fd;
 }
 
@@ -608,7 +616,6 @@ call_server(void)
 		FD_SET(adns_afd, &readfds);
 	    }
 
-#ifdef KLIPS
 	    if (kern_interface != NO_KERNEL)
 	    {
 		int fd = *kernel_ops->async_fdp;
@@ -620,7 +627,6 @@ call_server(void)
 		passert(!FD_ISSET(fd, &readfds));
 		FD_SET(fd, &readfds);
 	    }
-#endif
 
 	    if (listening)
 	    {
@@ -728,7 +734,6 @@ call_server(void)
 		ndes--;
 	    }
 
-#ifdef KLIPS
 	    if (kern_interface != NO_KERNEL
 		&& FD_ISSET(*kernel_ops->async_fdp, &readfds))
 	    {
@@ -739,7 +744,6 @@ call_server(void)
 		passert(GLOBALS_ARE_RESET());
 		ndes--;
 	    }
-#endif
 
 	    for (ifp = interfaces; ifp != NULL; ifp = ifp->next)
 	    {
@@ -779,12 +783,7 @@ call_server(void)
 #endif
 
 	    /* note we process helper things last on purpose */
-	    {
-		int helpers = pluto_crypto_helper_ready(&readfds);
-		DBG(DBG_CONTROL, DBG_log("* processed %d messages from cryptographic helpers\n", helpers));
-		
-		ndes -= helpers;
-	    }
+	    ndes -= pluto_crypto_helper_ready(&readfds);
 
 	    passert(ndes == 0);
 	}
